Package: demor
Title: Methods for Demographic Analysis
Version: 1.0.10
Authors@R: 
    person("Vadim", "Ustyuzhanin", , "vadim.v.ustyuzhanin@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-3800-1108"))
Description: Implements life tables, fertility and mortality indicators,
    decomposition methods, Lee-Carter mortality forecasting, Leslie
    matrices, and population pyramids for demographic analysis. Methods
    are described in Preston et al. (2001, ISBN:1557864519) and
    Ustyuzhanin (2025) <doi:10.17323/demreview.v12i4.30415>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: dplyr, forecast, ggplot2, magrittr, tidyr, scales, splines
Depends: R (>= 4.0.0)
LazyData: true
LazyDataCompression: gzip
LazyLoad: true
Suggests: knitr, rmarkdown, stringr, testthat
URL: https://vadvu.github.io/demor/, https://github.com/vadvu/demor
BugReports: https://github.com/vadvu/demor/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-18 16:41:59 UTC; vvust
Author: Vadim Ustyuzhanin [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3800-1108>)
Maintainer: Vadim Ustyuzhanin <vadim.v.ustyuzhanin@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-22 09:30:02 UTC
Built: R 4.5.2; ; 2026-05-25 02:06:39 UTC; unix

Package: demor
Title: Methods for Demographic Analysis
Version: 1.0.10
Authors@R: 
    person("Vadim", "Ustyuzhanin", , "vadim.v.ustyuzhanin@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-3800-1108"))
Description: Implements life tables, fertility and mortality indicators,
    decomposition methods, Lee-Carter mortality forecasting, Leslie
    matrices, and population pyramids for demographic analysis. Methods
    are described in Preston et al. (2001, ISBN:1557864519) and
    Ustyuzhanin (2025) <doi:10.17323/demreview.v12i4.30415>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: dplyr, forecast, ggplot2, magrittr, tidyr, scales, splines
Depends: R (>= 4.0.0)
LazyData: true
LazyDataCompression: gzip
LazyLoad: true
Suggests: knitr, rmarkdown, stringr, testthat
URL: https://vadvu.github.io/demor/, https://github.com/vadvu/demor
BugReports: https://github.com/vadvu/demor/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-18 16:41:59 UTC; vvust
Author: Vadim Ustyuzhanin [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3800-1108>)
Maintainer: Vadim Ustyuzhanin <vadim.v.ustyuzhanin@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-22 09:30:02 UTC
Built: R 4.5.2; ; 2026-05-22 11:23:31 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,mdecomp)
S3method(summary,leslie)
export("%>%")
export(LT)
export(MLT)
export(ages)
export(asdt)
export(ccm)
export(decomp)
export(edagger)
export(fert.approx)
export(get_rosbris)
export(gini)
export(hli)
export(leecart)
export(leslie)
export(mac)
export(mdecomp)
export(med.age)
export(mort.approx)
export(plot_pyr)
export(tatfr)
export(tfr)
export(yll)
import(dplyr)
import(forecast)
import(ggplot2)
import(scales)
import(splines)
import(tidyr)
importFrom(magrittr,"%>%")

<!-- README.md is generated from README.Rmd. Please edit that file -->

# demor

The goal of `demor` is to provide you with:  
1. the most basic functions for demographic analysis  
2. some data

Now, `demor` provides the ability to download Russian
mortality/fertility data from [Rosbris
(РосБРиС)](http://demogr.nes.ru/index.php/ru/demogr_indicat/data) in a
user-friendly format. It also includes functions for building life
tables, age and cause decompositions, fitting Lee-Carter model,
calculating years of life lost, and more.

### Installation

You can install the development version of `demor` from GitHub with:

``` r
# install.packages("devtools")
devtools::install_github("vadvu/demor")
```

### Tutorial

Examples of how to use `demor` can be found on its
[website](https://vadvu.github.io/demor/) in the [Get
started](https://vadvu.github.io/demor/articles/demor.html) section. The
detailed information about each function can be found in
[this](https://vadvu.github.io/demor/reference/index.html) section of
the website. 

The package is also presented in [this](https://doi.org/10.17323/demreview.v12i4.30415) paper (in Russ.).

### Suggested Citations

If you use the package in your research, please, cite this paper (in Russ.) 
where the package is presented:

**English citation**: Ustyuzhanin V. (2025). demor: R package for basic and advanced methods of demographic analysis. 
Demographic Review, 12(4), 100-122. 
https://doi.org/10.17323/demreview.v12i4.30415

**Russian citation**: Устюжанин В.В. (2025). demor: пакет в R для базовых и продвинутых методов демографического анализа. 
Демографическое обозрение, 12(4), 100-122. 
https://doi.org/10.17323/demreview.v12i4.30415

### Contact Information

If you have any comments on the `demor` (suggestions for development,
bug fixes, etc.), please, feel free to contact me via email:
<vvustiuzhanin@yandex.ru>

### Acknowledgements

This package is developed with support by the Russian Science Foundation (Project No 24-18-00650). 

Этот пакет подготовлен при поддержке Российского научного фонда (проект №24-18-00650).

#' Cohort Component Model for Projecting Population.
#'
#' @param Mx.f Matrix or dataframe with mx for females, where row represents age, column represents a forecasted period. Thus, first column should be the first projected period, while each row represents age-specific mortality rate.
#' @param Mx.m Optional. Matrix or dataframe with mx for males, where row represents age, column represents a forecasted period. Thus, first column should be the first projected period, while each row represents age-specific mortality rate.
#' @param Fx Matrix or dataframe with fx, where row represents age, column represents a forecasted period. Thus, first column should be the first projected period, while each row represents age-specific fertility rate.
#' @param Ix.f Optional. Matrix or dataframe with net number of female migrants, where row represents age, column represents a forecasted period. Thus, first column should be the first projected period, while each row represents net number of migrants.
#' @param Ix.m Optional. Matrix or dataframe with net number of male migrants, where row represents age, column represents a forecasted period. Thus, first column should be the first projected period, while each row represents net number of migrants.
#' @param age.mx Numeric vector. Age groups for Mx.f rows.
#' @param age.fx Numeric vector. Age groups for Fx rows.
#' @param N0.f Numeric vector. Female population in the initial period by age groups.
#' @param N0.m Optional. Numeric vector. Male population in the initial period by age groups.
#' @param srb Numeric. Sex ratio at birth for females. By default, it is 100/205.
#' @param ... Optional. Additional arguments for [leslie()] function.
#' @details
#' The model is calculated in matrix form as
#' \deqn{\mathbf{N}_{t+h} = \mathbf{L}_t(\mathbf{N}_{t} + \mathbf{I}_{t}/2) + \mathbf{I}_{t}/2}
#' where \eqn{\mathbf{N}_t} is a column vector of population for time \eqn{t} with \eqn{h} as a step of projection (it is the length of age interval), \eqn{\mathbf{L}_t} is Leslie matrix and \eqn{\mathbf{I}_{t}} is a column vector of net migration.
#'
#' Note that the model assumes that in \eqn{\mathbf{N}} all age intervals are the same (i.e. \eqn{age = \{0-1, 1-4, 5-9, ...\}} is not permitted). Fortunately (and thanks to me), the function handles such situations automatically by transforming all age groups into a unified standard.
#'
#' @return If `Mx.m` is `NULL`, a numeric matrix with projected female
#'   population, where rows are age groups and columns are periods `0:h`. If
#'   male mortality is supplied, a list with matrices `female`, `male`, and
#'   `all`, each with the same row/column structure.
#' @examples
#' age.mx <- seq(0, 80, 5)
#' age.fx <- seq(15, 45, 5)
#' Mx.f <- matrix(
#'   rep(seq(0.005, 0.12, length.out = length(age.mx)), 2),
#'   nrow = length(age.mx),
#'   ncol = 2
#' )
#' Fx <- matrix(
#'   rep(c(0.02, 0.08, 0.11, 0.09, 0.05, 0.02, 0.005), 2),
#'   nrow = length(age.fx),
#'   ncol = 2
#' )
#' N0.f <- round(100000 * exp(-0.04 * age.mx))
#' ccm(Mx.f = Mx.f, Fx = Fx, age.mx = age.mx, age.fx = age.fx, N0.f = N0.f)
#' @export
ccm <- function(Mx.f, Mx.m = NULL, Fx, Ix.f = NULL, Ix.m = NULL, age.mx, age.fx, N0.f, N0.m = NULL, srb = 100/205, ...){

  # checks
  if(ncol(Mx.f) != ncol(Fx)){
    stop("Number of periods (columns) in Mx.f and Fx differs, prepare data carefully")
  }else if(nrow(Mx.f) != length(N0.f)){
    stop("Number of age groups (rows) in Mx.f and N0 differs, prepare data carefully")
  }
  n <- length(age.mx)
  h <- ncol(Mx.f) - 1
  if(is.null(Ix.f)){
    Ix.f <- matrix(0, n, h)
  }else if (nrow(Mx.f) != nrow(Ix.f)){
    stop("Number of age groups (rows) in Mx and Ix differs, prepare data carefully")
  }else if (ncol(Mx.f) != ncol(Ix.f)){
    stop("Number of periods (columns) in Mx and Ix differs, prepare data carefully")
  }

  n0 <- length(age.mx)
  newage = NULL
  if(length(unique(diff(age.mx))) > 1){
    ux <- unique(diff(age.mx))
    agegroup <- ux[which.max(tabulate(match(diff(age.mx), ux)))]
    newage <- seq(0, max(age.mx), agegroup)
    newage0 <- newage
    newage0[1] = newage0[2] - 1

    N0.f <- cbind(N0.f, newage = demor::ages(x = age.mx, groups = newage0, char = FALSE))
    N0.f[which(N0.f[,"newage"] == newage0[1]), "newage"] <- newage[1]
    N0.f <- stats::aggregate(data = N0.f, N0.f ~ newage, FUN = sum)
    N0.f <- N0.f[,"N0.f"]
    if(!is.null(N0.m)){
      N0.m <- cbind(N0.m, newage = demor::ages(x = age.mx, groups = newage0, char = FALSE))
      N0.m[which(N0.m[,"newage"] == newage0[1]), "newage"] <- newage[1]
      N0.m <- stats::aggregate(data = N0.m, N0.m ~ newage, FUN = sum)
      N0.m <- N0.m[,"N0.m"]
    }

    Ix.f <- cbind(Ix.f, newage = demor::ages(x = age.mx, groups = newage0, char = FALSE))
    Ix.f <- apply(Ix.f, MARGIN = 2, FUN = function(x){
      ft <- cbind(x, newage = demor::ages(x = age.mx, groups = newage0, char = FALSE))
      ft[which(ft[,"newage"] == newage0[1]), "newage"] <- newage[1]
      st <- stats::aggregate(data = ft, x ~ newage, FUN = sum)
      st$x
    })
    if(is.null(Ix.m)){
      Ix.m <- Ix.f
      Ix.m <- matrix(0, nrow = nrow(Ix.f), ncol = ncol(Ix.f))
    }else{
      Ix.m <- cbind(Ix.m, newage = demor::ages(x = age.mx, groups = newage0, char = FALSE))
      Ix.m <- apply(Ix.m, MARGIN = 2, FUN = function(x){
        ft <- cbind(x, newage = demor::ages(x = age.mx, groups = newage0, char = FALSE))
        ft[which(ft[,"newage"] == newage0[1]), "newage"] <- newage[1]
        st <- stats::aggregate(data = ft, x ~ newage, FUN = sum)
        st$x
      })
    }
  }

  dat.f <- cbind(N0.f)
  for(t in 1:h){
    les.t <- demor::leslie(mx = Mx.f[,t], fx = Fx[,t], age.mx = age.mx, age.fx = age.fx, srb = srb, sex = "f", ...)
    dat.f = cbind(dat.f, N0.f)
    dat.f[,t+1] = les.t %*% (dat.f[,t] + Ix.f[,t]/2) + Ix.f[,t]/2
  }


  if(!is.null(Mx.m)){

    if(is.null(Ix.m)){
      Ix.m <- Ix.f
      Ix.m <- matrix(0, nrow = nrow(Ix.f), ncol = ncol(Ix.f))
    }

    dat.m <- cbind(N0.m)
    for(t in 1:h){
      les.t <- demor::leslie(mx = Mx.m[,t], fx = Fx[,t], age.mx = age.mx, age.fx = age.fx, srb = 1-srb, sex = "m", ...)
      B.t <- les.t[1,] %*% (dat.f[,t] + Ix.f[,t]/2)
      les.t[1,] <- 0
      dat.m = cbind(dat.m, N0.m)
      dat.m[,t+1] = les.t %*% (dat.m[,t] + Ix.m[,t]/2) + Ix.m[,t]/2
      dat.m[1,t+1] <- B.t
    }
  }

  if(is.null(Mx.m)){
    rownames(dat.f) <- if(is.null(newage)) age.mx else newage
    colnames(dat.f) <- 0:h
    return(dat.f)
  }else{
    rownames(dat.f) <- rownames(dat.m) <- if(is.null(newage)) age.mx else newage
    colnames(dat.f) <- colnames(dat.m) <- 0:h
    list(
      female = dat.f,
      male = dat.m,
      all = dat.f + dat.m
    )
  }

}

#' Leslie Matrix
#'
#' @param mx Numeric array of age specific mortality rates.
#' @param fx Numeric array of age specific fertility rates.
#' @param age.mx Numeric array of ages for mx.
#' @param age.fx Numeric array of ages for fx.
#' @param srb Numeric. Sex ratio at birth. Usually it is assumed that for males it is `105/205`, for females it is `100/205`. By default, it is `100/205`.
#' @param fin Logical. Should the survival rate for the last age-group be nonzero? By default it is `FALSE`, so the last survival rate is 0 as in classical model. Otherwise, it is \eqn{T_{x}/T_{x-1}}.
#' @param ... Optional. Additional arguments for [LT()] function.
#' @seealso [summary.leslie()] for `leslie` output that calculates \eqn{\lambda}, \eqn{r}, \eqn{w} and \eqn{v}.
#' @return A square numeric matrix of class `leslie` with one row and one
#'   column per age group. The first row contains fertility contributions and
#'   the subdiagonal contains survival ratios.
#' @examples
#' mx <- c(0.02, 0.01, 0.012, 0.015, 0.02)
#' fx <- c(0.05, 0.08)
#' leslie(mx = mx, fx = fx, age.mx = 0:4, age.fx = 1:2)
#' @export
leslie <- function(mx, fx, age.mx, age.fx, srb = 100/205, fin = TRUE, ...){
  lt <- LT(age = age.mx, mx = mx, ...)
  radix <- lt[1,"lx"]
  Tx <- lt[,"Tx"]
  n0 <- length(age.mx)
  if(length(unique(diff(age.mx))) > 1){
    ux <- unique(diff(age.mx))
    agegroup <- ux[which.max(tabulate(match(diff(age.mx), ux)))]
    newage <- seq(0, max(age.mx), agegroup)
    newage0 <- newage
    newage0[1] = newage0[2] - 1
    lt <- cbind(lt, newage = demor::ages(x = age.mx, groups = newage0, char = FALSE))
    lt[which(lt[,"newage"] == newage0[1]), "newage"] <- newage[1]
    lt <- stats::aggregate(data = lt, Lx ~ newage, FUN = sum)
    age.mx = newage
  }
  n <- length(age.mx)

  Lx = lt[,"Lx"]
  Sx <- Lx[-1] / Lx[-n]
  Sx[n-1] <- Tx[n0]/Tx[n0-1]
  if(any(is.na(Sx))){
    Sx[is.na(Sx)] = 0
    warning("There are NAs in survival rates, change them to 0")
  }

  A <- matrix(0, n, n)
  A[n,n] <- ifelse(fin, Sx[n-1], 0)
  diag(A[-1,1:(n-1)]) <- Sx[-n]
  names(fx) <- age.fx
  fert = sapply(age.mx, FUN = function(x) ifelse(x %in% age.fx, fx[paste0(x)], 0) )
  k <- srb * ( Lx[1] / (2*radix) )
  A[1,] <- k * ( fert + c(fert[-1] * Sx[-n], 0) )

  class(A) <- "leslie"
  A
}

#' Fertility models for ASFR approximation
#'
#' @param fx Numeric vector of age specific fertility rates.
#' @param age Numeric vector of ages.
#' @param model Character. Model name to be estimated. Now "Hadwiger", "Gamma", "Brass" and "Beta" are supported.
#' @param se Logical. Should bootstrapped variance for ASFR approximation be calculated. Default is `FALSE` for no bootstrap.
#' @param start Numeric vector with user-specific values of parameters for optimization. Default is `NULL` (choose automatically)
#' @param alpha Numeric. Used if `se = TRUE`, the level of uncertainty. By default, `alpha = 0.05` for 95% CI.
#' @param bn Numeric. Used if `se = TRUE`, number of bootstrap samples. By default, `bn = 1000`.
#' @return A list with two components: `model`, a list describing the fitted
#'   fertility model (`type`, fitted `params`, `rmse`, and, if `se = TRUE`,
#'   bootstrap `covmat` and parameter percentile intervals `prc`); and
#'   `predicted`, a data frame with observed and fitted age-specific fertility
#'   rates. When `se = TRUE`, `predicted` also includes bootstrap standard
#'   errors and percentile intervals.
#'
#' @details
#' This function runs least squares optimization (using default `optim`) of the selected fertility function with 1e-06 as tolerance parameter.
#'
#' \eqn{f_x} is age-specific fertility rate for age \eqn{x}.
#'
#' ## Hadwiger model
#' The model is as follows: \deqn{f_x = \frac{ab}{c} \frac{c}{x}^{3/2} exp[-b^2(\frac{c}{x}+\frac{x}{c}-2)]}
#' where \eqn{a,b,c} are estimated parameters that do not have demographic interpretation. Sometimes \eqn{c} is interpreted as mean age at childbearing.
#'
#' ## Gamma model
#' The model is as follows: \deqn{f_x = \frac{R}{\Gamma(b)c^b}(x-d)^{b-1} exp[-(\frac{x-d}{c})]}
#' where \eqn{R,b,c,d} are estimated parameters. \eqn{\Gamma} is gamma function. \eqn{R} can be interpreted as fertility level (TFR) and \eqn{d} as mean age at childbearing.
#'
#' ## Brass model
#' The model is as follows: \deqn{f_x = c(x-d)(d+w-x)}
#' where \eqn{c,d,w} are estimated parameters.
#'
#' ## Beta model
#' The model is as follows: \deqn{f_x = \frac{R}{B(A,C)}(\beta - \alpha)^{-(A+C-1)}(x-\alpha)^{(A-1)}(\beta-x)^{(B-1)}}
#' where \eqn{B(A, C)} is beta function, \eqn{R, \beta, \alpha} are estimated parameters, which can be interpreted as fertility level (TFR) and max and min age of childbearing respectively.
#' \eqn{A,C} are
#' \deqn{C = (\frac{(v - \alpha)(\beta - v)}{\tau^2} - 1)\frac{\beta - v}{\beta - \alpha}}
#' \deqn{A = C\frac{v-\alpha}{v - \beta}}
#' where \eqn{v, \tau^2} are estimated parameters, where \eqn{v} can be interpreted as mean age at childbearing. Thus, Beta model uses 5 parameters \eqn{R, \beta, \alpha, v, \tau^2}, where only \eqn{\tau^2} has no demographic interpretation.
#'
#' @references
#' Peristera, P., & Kostaki, A. (2007). Modeling fertility in modern populations. *Demographic Research*, *16*, 141-194.
#'
#' @export
#'
#' @examples
#' age <- seq(15, 45, 5)
#' fx <- c(0.03, 0.10, 0.14, 0.12, 0.07, 0.03, 0.01)
#' fert.approx(fx = fx, age = age, model = "Hadwiger", se = FALSE)
fert.approx <- function(fx, age, model, start = NULL, se = FALSE, alpha = 0.05, bn = 1000){

  ##### Checks

  if(length(age) != length(fx)){
    stop("age and fx do not have the same length")
  }
  if(!(model %in% c("Hadwiger", "Gamma", "Brass", "Beta"))){
    stop("model can be only 'Hadwiger', 'Gamma', 'Brass' or 'Beta'")
  }

  ##### Models

  hadw <- function(pars, age){
    a = pars[1]; b = pars[2]; c = pars[3]
    fx.pred = (a*b/c)*(c/age)^(1.5) * exp(-b^2*(c/age + age/c - 2))
    fx.pred
  }
  mygamma <- function(pars, age){
    R = pars[1]; b = pars[2]; c = pars[3]; d = pars[4]
    fx.pred = R*( 1/( gamma(b)*c^b ) ) * (age-d)^(b-1) * exp( -1*( (age-d)/c ) )
    fx.pred
  }
  brass <- function(pars, age){
    c = pars[1]; d = pars[2]; w = pars[3]
    fx.pred = c*(age - d)*(d+w-age)^2
    fx.pred
  }
  mybeta <- function(pars, age){
    a = pars[1]; b = pars[2]; v = pars[3]; tau.sq = pars[4]; R = pars[5]
    B = ((v-a)*(b-v)/tau.sq - 1)*((b-v)/(b-a))
    A = B*(v - a)/(b-v)
    fx.pred =  R*(1/beta(A, B)) * (b - a)^-(A+B-1) * (age-a)^(A-1) * (b-age)^(B-1)
    fx.pred
  }

  ##### Starting parameters

  if(!is.null(start)){
    first.pars <- start
  }else if (model == "Hadwiger"){
    first.pars <- c(tfr(fx, unique(diff(age))),
                    (max(fx)*mac(fx, age))/tfr(fx, unique(diff(age))),
                    mac(fx, age)) #a, b, c
  }else if(model == "Gamma"){
    first.pars <- c(tfr(fx, unique(diff(age))), 1, 1, min(age)) #R, b, c ,d
  }else if(model == "Brass"){
    first.pars <- c(1, min(age), max(age)-min(age)) #c, d, w
  }else if(model == "Beta"){
    first.pars <- c(min(age), max(age), mac(fx, age), mac(fx, age)+2, tfr(fx, unique(diff(age)))) #a, b, v, tau.sq, R
  }

  ##### Estimator

  estimate <- function(mf, pars, fx, age){

    objective <- function(pars, fx, age) {
      fx.pred <- mf(pars, age = age)
      sum((fx-fx.pred)^2)
    }

    m1 <- stats::optim(par = pars,
                fn = objective, fx = fx, age = age,
                control =  list(maxit = 1e6))
    m2 <- stats::optim(par = m1$par,
                fn = objective, fx = fx, age = age,
                control =  list(maxit = 1e6))

    while(m1$value - m2$value > 1e-06){
      m1 <- m2
      m2 <- stats::optim(par = m1$par,
                  fn = objective, fx = fx, age = age,
                  control =  list(maxit = 1e6))
    }
    m2
  }


  ##### Results
  if(model == "Brass"){
    ffn <- brass
    nms <- c("c", "d", "w")
  }else if(model == "Hadwiger"){
    ffn <- hadw
    nms <- c("a", "b", "c")
  }else if(model == "Gamma"){
    ffn <- mygamma
    nms <- c("R", "b", "c" ,"d")
  }else if(model == "Beta"){
    ffn <- mybeta
    nms <- c("a", "b", "v", "tau.sq", "R")
  }
  est <- estimate(mf = ffn, first.pars, fx = fx, age = age)
  names(est$par) <- nms

  #### SE
  if(se == TRUE){
    for(i in 1:bn){
      samplei <- sample(1:length(fx), length(fx), replace = TRUE)
      esti = estimate(mf = ffn, first.pars, fx = fx[samplei], age = age[samplei])$par
      if(i == 1){
        bootpars = matrix(esti, nrow = 1)
        bootpred = matrix(ffn(esti, age = age), ncol = 1)
      }else{
        bootpars = rbind(bootpars, esti)
        bootpred = cbind(bootpred, ffn(esti, age = age))
      }
    }
    covmat = stats::cov(bootpars)
    prc.pars = apply(bootpars, 2, stats::quantile, probs = alpha/2, na.rm = TRUE)
    prc.pars = t(rbind(prc.pars, apply(bootpars, 2, stats::quantile, probs = 1-alpha/2, na.rm = TRUE)))
    rownames(prc.pars) <- nms
    colnames(prc.pars) <- c("prc.low", "prc.high")
    se.pred = diag(stats::var(t(bootpred), na.rm = TRUE))^0.5

    return(list(
      model = list(
        type = model,
        params = est$par,
        covmat = covmat,
        prc = prc.pars,
        rmse = sqrt(sum((ffn(est$par, age = age) - fx)^2) / (length(age) - length(est$par)))
      ),
      predicted = data.frame(age = age, fx.model = ffn(est$par, age = age), fx = fx, se = se.pred,
                             prc.low = apply(t(bootpred), 2, stats::quantile, probs = alpha/2, na.rm = TRUE),
                             prc.high = apply(t(bootpred), 2, stats::quantile, probs = 1-alpha/2, na.rm = TRUE)
      )
    )
    )
  }else{
    return(list(
      model = list(
        type = model,
        params = est$par,
        rmse = sqrt(sum((ffn(est$par, age = age) - fx)^2) / (length(age) - length(est$par)))
      ),
      predicted = data.frame(age = age, fx.model = ffn(est$par, age = age), fx = fx)
    )
    )
  }
}

#' Median age calculation
#'
#' @param N Numeric array. Population counts by age groups (from young to old)
#' @param age Numeric array. Lower bounds of age groups, same length as `N`
#'
#' @return A length-1 numeric value giving the estimated median age of the
#'   population represented by `N`.
#' @examples
#' N <- c(100, 90, 80, 70, 60)
#' age <- seq(0, 20, 5)
#' med.age(N, age)
#' @export
med.age <- function(N, age){
  if(length(age) != length(N)){
    stop("lengths of age groups (age) and Population counts (N) are not the same")
  }
  cN <- cumsum(N)
  m <- cN[length(N)]/2
  inter <- which(cN >= m)[1]
  prev_cN <- ifelse(inter == 1, 0, cN[inter - 1])
  int <- ifelse(inter == length(age), max(diff(age)), age[inter+1] - age[inter])
  fin <- age[inter] + (m - prev_cN) / N[inter] * int
  return(round(fin, 2))
}

test_that("med.age errors on length mismatch", {
  expect_error(med.age(1:3, c(0, 5)), "lengths of age groups \\(age\\) and Population counts \\(N\\) are not the same")
})

test_that("med.age works", {
  age <- 0:2
  N <- c(10, 10, 10)
  expect_equal(med.age(N, age), 1.5)
})

#' Leslie matrix summary
#'
#' @param object A leslie matrix from [leslie()].
#' @param d Time step. By default it is 1. Only affects \eqn{r}.
#' @param ... Ignored.
#' @details
#' The function calculates \eqn{\lambda}, \eqn{r}, \eqn{w} and \eqn{v}.
#' \itemize{
#'    \item \eqn{\lambda} -- asymptotic growth factor that is dominant eigenvalue.
#'    \item \eqn{r} -- asymptotic growth rate that is \eqn{ln \lambda / \delta t}.
#'    \item \eqn{w} -- stable age distribution normalized to 1 s.t. \eqn{\sum_x^{\omega} w_x = 1} where \eqn{x} is age.
#'    \item \eqn{v} -- reproductive values normalized s.t. \eqn{v'w = 1}.
#' }
#' @return A list of class `summary.leslie` with four components:
#'   `lambda` (dominant eigenvalue / asymptotic growth factor), `r`
#'   (intrinsic growth rate), `w` (stable age distribution summing to 1),
#'   and `v` (reproductive values normalized so that `sum(v * w) = 1`).
#'
#' @examples
#' mx <- c(0.02, 0.01, 0.012, 0.015, 0.02)
#' fx <- c(0.05, 0.08)
#' les <- leslie(mx = mx, fx = fx, age.mx = 0:4, age.fx = 1:2)
#' summary(les)
#'
#' @seealso [leslie()]
#' @export
#' @method summary leslie
summary.leslie <- function(object, d = 1, ...){
  ev <- eigen(object)
  k <- which.max(Re(ev$values))

  # lambda = exp growth rate
  lambda <- Re(ev$values[k])

  # stable age dist
  w <- Re(ev$vectors[,k])
  w <- w / sum(w)

  # reproductive value
  evL <- eigen(t(object))
  kL <- which.max(Re(evL$values))
  v <- Re(evL$vectors[,kL])
  v <- v / as.numeric(t(v) %*% w)

  out <- list(
    lambda = lambda,
    r = log(lambda)/d,
    w = w,
    v = v
    )
  class(out) <- "summary.leslie"
  out
}

#' Download RosBris Data
#'
#' Download age-specific mortality, fertility, and population data from the
#' Russian Fertility and Mortality Database (RosBris) and return them as
#' long-format data frames for use in `demor`.
#'
#' @param dataset Character. Dataset to download and parse. One of
#'   `"mortality_1"`, `"mortality_5"`, `"fertility_1"`, or `"fertility_5"`.
#' @param refresh Logical. If `FALSE` (default), previously downloaded RosBris
#'   archives are reused from the local cache directory. If `TRUE`, archives are
#'   downloaded again from the RosBris website.
#'
#' @details
#' The function downloads official `.zip` archives from the RosBris website,
#' stores them in a user cache directory created by `tools::R_user_dir()`, reads
#' `.txt` tables from the archives, and converts them to long-format data frames.
#'
#' Returned data have the following structure:
#'
#' - `"mortality_1"`: `year`, `code`, `territory`, `sex`, `age`, `mx`, `N`,
#'   `Dx`, `name`.
#' - `"mortality_5"`: `year`, `code`, `territory`, `sex`, `age`, `mx`, `N`,
#'   `Dx`, `name`.
#' - `"fertility_1"`: `year`, `code`, `territory`, `age`, `fx`, `N`, `Bx`,
#'   `name`.
#' - `"fertility_5"`: `year`, `code`, `territory`, `age`, `fx`, `fx1`, `fx2`,
#'   `fx3`, `fx4`, `fx5`, `N`, `Bx`, `Bx1`, `Bx2`, `Bx3`, `Bx4`, `Bx5`,
#'   `name`.
#'
#' At the moment, `get_rosbris()` uses the legacy RosBris series corresponding
#' to the periods `1989-2014` and `2015-2022`. Updated post-census series are
#' not used by this function.
#'
#' @return A data frame with one row per year-region-age combination. Column
#'   structure depends on `dataset`; see **Details**.
#'
#' @seealso [rosbris.codes]
#'
#' @source Russian Fertility and Mortality Database. Center for Demographic
#'   Research, Moscow (Russia). Available at
#'   <https://www.nes.ru/research-main/research-centers/demogr/demogr-fermort-data>
#'
#' @examples
#' \donttest{
#' if (interactive()) {
#'   mort <- get_rosbris("mortality_5")
#'   rus2010 <- subset(
#'     mort,
#'     year == 2010 & code == 1100 & sex == "m" & territory == "t"
#'   )
#'   head(rus2010)
#' }
#' }
#'
#' @export
get_rosbris <- function(dataset = c("mortality_1", "mortality_5", "fertility_1", "fertility_5"),
                        refresh = FALSE) {
  dataset <- match.arg(dataset)

  switch(
    dataset,
    mortality_1 = .rosbris_mortality_1(refresh = refresh),
    mortality_5 = .rosbris_mortality_5(refresh = refresh),
    fertility_1 = .rosbris_fertility_1(refresh = refresh),
    fertility_5 = .rosbris_fertility_5(refresh = refresh)
  )
}


.rosbris_zip_csv <- function(url, refresh = FALSE) {
  cache_dir <- file.path(tools::R_user_dir("demor", which = "cache"), "rosbris")
  dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)

  zip_path <- file.path(cache_dir, basename(url))

  if (!file.exists(zip_path) || isTRUE(refresh)) {
    utils::download.file(
      url = url,
      destfile = zip_path,
      mode = "wb",
      quiet = TRUE
    )
  }

  members <- utils::unzip(zip_path, list = TRUE)
  txt_member <- members$Name[grepl("\\.txt$", members$Name, ignore.case = TRUE)][1]

  con <- unz(zip_path, txt_member)
  on.exit(try(close(con), silent = TRUE), add = TRUE)

  utils::read.csv(con, check.names = FALSE)
}


.rosbris_num <- function(x) {
  suppressWarnings(as.numeric(x))
}


.rosbris_codes <- function() {
  env <- new.env(parent = emptyenv())
  utils::data("rosbris.codes", package = "demor", envir = env)
  env[["rosbris.codes"]]
}


.rosbris_add_names <- function(data) {
  codes <- .rosbris_codes()

  if (is.null(codes)) {
    return(data)
  }

  dplyr::left_join(data, codes[, c("code", "name")], by = "code")
}


.rosbris_extract_br5_from_order <- function(data_order) {
  keep <- c("Year", "Reg", "Group", grep("^BrOAa", names(data_order), value = TRUE))
  out <- data_order[, keep, drop = FALSE]
  names(out) <- sub("^BrOAa", "Br5a", names(out))
  out
}


.rosbris_mortality_1 <- function(refresh = FALSE) {
  data1m <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/02/001-1-DRa1989-2014.zip",
    refresh = refresh
  )
  data2m <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/05/DRa2015-2022.zip",
    refresh = refresh
  )
  datam <- rbind(data1m, data2m)

  data1p <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/03/005-1-PopDa1989-2014.zip",
    refresh = refresh
  )
  data2p <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/08/PopDa2015-2022.zip",
    refresh = refresh
  )
  datap <- rbind(data1p, data2p)

  datap <- datap %>%
    tidyr::pivot_longer(cols = dplyr::starts_with("Pop"), names_to = "age", values_to = "N") %>%
    dplyr::transmute(year = Year, code = Reg, sex = Sex, age, N, territory = Group) %>%
    dplyr::mutate(territory = tolower(territory), sex = tolower(sex)) %>%
    dplyr::mutate(age = gsub("PopDa", "", age)) %>%
    dplyr::mutate(
      year = .rosbris_num(year),
      code = .rosbris_num(code),
      N = .rosbris_num(N),
      age = .rosbris_num(age)
    )

  datam <- datam %>%
    tidyr::pivot_longer(cols = dplyr::starts_with("Dr"), names_to = "age", values_to = "mx") %>%
    dplyr::transmute(year = Year, code = Reg, territory = Group, sex = Sex, age, mx) %>%
    dplyr::mutate(territory = tolower(territory), sex = tolower(sex)) %>%
    dplyr::mutate(age = gsub("Dra", "", age)) %>%
    dplyr::mutate(
      year = .rosbris_num(year),
      code = .rosbris_num(code),
      mx = .rosbris_num(mx),
      age = .rosbris_num(age)
    ) %>%
    dplyr::mutate(mx = mx / 1000000)

  data <- merge(datam, datap) %>%
    dplyr::mutate(Dx = round(mx * N, 2))

  data$age <- .rosbris_num(data$age)
  data <- data %>%
    dplyr::arrange(code, year, territory, sex, age) %>%
    tidyr::drop_na(code)

  .rosbris_add_names(data)
}


.rosbris_mortality_5 <- function(refresh = FALSE) {
  data1m <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/02/003-1-DR5a1989-2014.zip",
    refresh = refresh
  )
  data2m <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/05/DR5a2015-2022.zip",
    refresh = refresh
  )
  datam <- rbind(data1m, data2m)

  data1p <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/03/007-1-PopD5a1989-2014.zip",
    refresh = refresh
  )
  data2p <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/05/PopD5a2015-2022.zip",
    refresh = refresh
  )
  datap <- rbind(data1p, data2p)

  datap <- datap %>%
    tidyr::pivot_longer(cols = dplyr::starts_with("Pop"), names_to = "age", values_to = "N") %>%
    dplyr::transmute(year = Year, code = Reg, sex = Sex, age, N, territory = Group) %>%
    dplyr::mutate(territory = tolower(territory), sex = tolower(sex)) %>%
    dplyr::mutate(age = gsub("PopD5a", "", age)) %>%
    dplyr::mutate(
      year = .rosbris_num(year),
      code = .rosbris_num(code),
      N = .rosbris_num(N),
      age = .rosbris_num(age)
    )

  datam <- datam %>%
    tidyr::pivot_longer(cols = dplyr::starts_with("Dr"), names_to = "age", values_to = "mx") %>%
    dplyr::transmute(year = Year, code = Reg, territory = Group, sex = Sex, age, mx) %>%
    dplyr::mutate(territory = tolower(territory), sex = tolower(sex)) %>%
    dplyr::mutate(age = gsub("DrAa", "", age)) %>%
    dplyr::mutate(
      year = .rosbris_num(year),
      code = .rosbris_num(code),
      mx = .rosbris_num(mx),
      age = .rosbris_num(age)
    ) %>%
    dplyr::mutate(mx = mx / 1000000)

  data <- merge(datam, datap) %>%
    dplyr::mutate(Dx = round(mx * N, 2))

  data$age <- .rosbris_num(data$age)
  data <- data %>%
    dplyr::arrange(code, year, territory, sex, age) %>%
    tidyr::drop_na(code)

  .rosbris_add_names(data)
}


.rosbris_fertility_1 <- function(refresh = FALSE) {
  data1f <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/01/02-1-BRa1989-2014.zip",
    refresh = refresh
  )
  data2f <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/05/BRa2015-2022.zip",
    refresh = refresh
  )
  dataf <- rbind(data1f, data2f)

  data1p <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/03/010-1-PopBa1989-2014.zip",
    refresh = refresh
  )
  data2p <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/05/PopBa2015-2022.zip",
    refresh = refresh
  )
  datap <- rbind(data1p, data2p)

  datap <- datap %>%
    tidyr::pivot_longer(cols = dplyr::starts_with("Pop"), names_to = "age", values_to = "N") %>%
    dplyr::transmute(year = Year, code = Reg, age, N, territory = Group) %>%
    dplyr::mutate(territory = tolower(territory)) %>%
    dplyr::mutate(age = gsub("PopBa", "", age)) %>%
    dplyr::mutate(
      year = .rosbris_num(year),
      code = .rosbris_num(code),
      N = .rosbris_num(N),
      age = .rosbris_num(age)
    )

  dataf <- dataf %>%
    tidyr::pivot_longer(cols = dplyr::starts_with("Br"), names_to = "age", values_to = "fx") %>%
    dplyr::transmute(year = Year, code = Reg, territory = Group, age, fx) %>%
    dplyr::mutate(territory = tolower(territory)) %>%
    dplyr::mutate(age = gsub("Bra", "", age)) %>%
    dplyr::mutate(
      year = .rosbris_num(year),
      code = .rosbris_num(code),
      fx = .rosbris_num(fx),
      age = .rosbris_num(age)
    ) %>%
    dplyr::mutate(fx = fx / 1000000)

  data <- merge(dataf, datap) %>%
    dplyr::mutate(Bx = round(fx * N, 2))

  data$age <- .rosbris_num(data$age)
  data <- data %>%
    dplyr::arrange(code, year, territory, age) %>%
    tidyr::drop_na(code)

  .rosbris_add_names(data)
}


.rosbris_fertility_5 <- function(refresh = FALSE) {
  data1f <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/01/05-1-BR5a1989-2014.zip",
    refresh = refresh
  )
  data1fp <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/01/07-1-BR5aO1989-2014.zip",
    refresh = refresh
  )
  data2fp <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/05/BR5aO2015-2022.zip",
    refresh = refresh
  )

  data2f <- .rosbris_extract_br5_from_order(data2fp)
  dataf <- rbind(data1f, data2f)
  datafp <- rbind(data1fp, data2fp)

  data1p <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/03/012-1-PopB5a1989-2014.zip",
    refresh = refresh
  )
  data2p <- .rosbris_zip_csv(
    "https://www.nes.ru/files/research/demogr/data-rus/05/PopB5a2015-2022.zip",
    refresh = refresh
  )
  datap <- rbind(data1p, data2p)

  datap <- datap %>%
    tidyr::pivot_longer(cols = dplyr::starts_with("Pop"), names_to = "age", values_to = "N") %>%
    dplyr::transmute(year = Year, code = Reg, age, N, territory = Group) %>%
    dplyr::mutate(territory = tolower(territory)) %>%
    dplyr::mutate(age = gsub("PopB5a", "", age)) %>%
    dplyr::mutate(
      year = .rosbris_num(year),
      code = .rosbris_num(code),
      N = .rosbris_num(N),
      age = .rosbris_num(age)
    )

  dataf <- dataf %>%
    tidyr::pivot_longer(cols = dplyr::starts_with("Br"), names_to = "age", values_to = "fx") %>%
    dplyr::transmute(year = Year, code = Reg, territory = Group, age, fx) %>%
    dplyr::mutate(territory = tolower(territory)) %>%
    dplyr::mutate(age = gsub("Br5a", "", age)) %>%
    dplyr::mutate(
      year = .rosbris_num(year),
      code = .rosbris_num(code),
      fx = .rosbris_num(fx),
      age = .rosbris_num(age)
    ) %>%
    dplyr::mutate(fx = fx / 1000000)

  datafp <- datafp %>%
    tidyr::pivot_longer(cols = dplyr::starts_with("Br"), names_to = "age", values_to = "fx") %>%
    dplyr::transmute(year = Year, code = Reg, territory = Group, age, fx) %>%
    dplyr::mutate(territory = tolower(territory)) %>%
    dplyr::mutate(age = gsub("BrO", "", age)) %>%
    dplyr::mutate(age = gsub("a", "", age)) %>%
    dplyr::mutate(parity = substring(age, 1, 1), age = substring(age, 2)) %>%
    dplyr::mutate(
      year = .rosbris_num(year),
      code = .rosbris_num(code),
      fx = .rosbris_num(fx),
      age = .rosbris_num(age)
    ) %>%
    dplyr::mutate(fx = fx / 1000000) %>%
    dplyr::mutate(parity = paste0("fx", parity)) %>%
    tidyr::pivot_wider(names_from = parity, values_from = fx) %>%
    dplyr::select(-fxA)

  dataf <- dplyr::left_join(dataf, datafp, by = c("code", "age", "territory", "year"))

  data <- dplyr::left_join(dataf, datap, by = c("code", "age", "territory", "year")) %>%
    dplyr::mutate(Bx = round(fx * N, 2))

  data$age <- .rosbris_num(data$age)
  data <- data %>%
    dplyr::arrange(code, year, territory, age) %>%
    tidyr::drop_na(code)

  for (i in 1:5) {
    data[[paste0("Bx", i)]] <- round(data[[paste0("fx", i)]] * data[["N"]], 2)
  }

  .rosbris_add_names(data)
}

#' Age decomposition of mortality: single decrement process
#'
#' @param mx1 Numeric array with age specific mortality rates of population 1 (base population).
#' @param mx2 Numeric array with age specific mortality rates of population 2 (compared population).
#' @param sex Character. Sex. "m" for males and "f" for females. By default, `sex = "m"`.
#' @param age Numeric array of age intervals; for full life table = `0:100`; for concise life table = `c(0:1, seq(5,85,5))`
#' @param method Character. Decomposition method. "andreev" (1982), "arriaga" (1984) or "pollard" (1982) - slightly different in their results. By default, `method = "andreev"`.
#' @param ax1 Optional. Numeric array with ax for the 1st population. By default, it is a the middle of the interval, while ax for age 0 is modeled as in Andreev & Kingkade (2015).
#' @param ax2 Optional. Numeric array with ax for the 2nd population. By default, it is a the middle of the interval, while ax for age 0 is modeled as in Andreev & Kingkade (2015).
#' @details
#' Example of decomposition using Andreev (1982) formulas:
#' \deqn{\Delta_x = l_x^2(e_x^2 - e_x^1) - l_{x+n}^2(e_{x+n}^2 - e_{x+n}^1), \ \ \ x \neq \omega}
#' \deqn{\Delta_{\omega} = l_{\omega}^2(e_{\omega}^2 - e_{\omega}^1)}
#' where \eqn{\Delta_x} is an absolute contribution of age \eqn{x} to difference in \eqn{e_0} between the second and the first population. \eqn{e_x^i, l_x^i} are life table functions for population \eqn{i}. \eqn{\omega} is the last age group. Note, \eqn{e_0^2 - e_0^1 = \sum_{x}^{\omega}\Delta_x}
#'
#' @seealso [mdecomp()] for age and cause decomposition
#' @references
#' 1. Arriaga, E. E. (1984). Measuring and explaining the change in life expectancies. *Demography*, *21*, 83-96.
#' 2. Andreev, E. M. (1982). Metod komponent v analize prodolzhitel'nosti zhizni. *Vestnik statistiki*, *9*, 42-47.
#' 3. Pollard, J. H. (1982). The expectation of life and its relationship to mortality. *Journal of the Institute of Actuaries*, *109(2)*, 225–240.
#' @import dplyr
#' @return A data frame with one row per age group. It always contains `age`,
#'   `ex12`, and `ex12_prc`, where `ex12` is the absolute age contribution to
#'   the life-expectancy difference and `ex12_prc` is the percentage
#'   contribution. Additional columns depend on `method` and contain
#'   intermediate life-table quantities used in the decomposition.
#' @examples
#' age <- 0:5
#' mx1 <- c(0.02, 0.01, 0.012, 0.015, 0.02, 0.03)
#' mx2 <- c(0.018, 0.009, 0.011, 0.014, 0.019, 0.028)
#' decomp(mx1, mx2, age = age)$ex12
#' @export
decomp <- function(mx1, mx2, sex = "m", age, method = "andreev", ax1 = NULL, ax2 = NULL) {
  if (!(method %in% c("andreev", "arriaga", "pollard"))){stop("Choose method (misspelling?)")}
  lt1 <- LT(age = age, sex = sex, mx = mx1, ax = ax1)
  lt2 <- LT(age = age, sex = sex, mx = mx2, ax = ax2)
  if (method == "andreev") {
    comp = data.frame(
      age = age,
      ex1 = lt1[, "ex"],
      ex2 = lt2[, "ex"],
      lx2 = lt2[, "lx"]
    )
    comp$dex = dex_1 = comp$ex2 - comp$ex1
    c1_1 = lt2[,"lx"] * dex_1
    ex12_1 = round(c(rev(diff(rev(c1_1))), c1_1[length(c1_1)]), 2)

    dex_2 = comp$ex1 - comp$ex2
    c1_2 = lt1[,"lx"] * dex_2
    ex12_2 = round(c(rev(diff(rev(c1_2))), c1_2[length(c1_2)]), 2)

    comp$ex12 = round(0.5*(ex12_1 - ex12_2), 2)
    comp$ex12_prc = 100 * comp$ex12 / sum(comp$ex12)

  } else if (method == "arriaga"){
    comp = data.frame(
      age = age,
      l1 = lt1[, "lx"],
      L1 = lt1[, "Lx"],
      T1 = lt1[, "Tx"],
      l2 = lt2[, "lx"],
      L2 = lt2[, "Lx"],
      T2 = lt2[, "Tx"]
    )
    comp$ex12 = ex12_1 = ex12_2 = NA
    ex12_1 = ( lt1[,"lx"] / lt1[1,"lx"] ) * ( lt2[,"Lx"] / lt2[,"lx"] - lt1[,"Lx"] / lt1[,"lx"] ) +
      ( dplyr::lead(lt2[,"Tx"]) / lt1[1,"lx"] ) * ( lt1[,"lx"] / lt2[,"lx"] - dplyr::lead(lt1[,"lx"] / lt2[,"lx"]) )
    ex12_1[length(ex12_1)] = (lt1[nrow(lt1),"lx"] / lt1[1,"lx"]) * ( lt2[nrow(lt1),"Lx"] / lt2[nrow(lt1),"lx"] - lt1[nrow(lt1),"Lx"] / lt1[nrow(lt1),"lx"] )

    lt10 <- lt2
    lt2 <- lt1
    lt1 <- lt10
    ex12_2 = ( lt1[,"lx"] / lt1[1,"lx"] ) * ( lt2[,"Lx"] / lt2[,"lx"] - lt1[,"Lx"] / lt1[,"lx"] ) +
      ( dplyr::lead(lt2[,"Tx"]) / lt1[1,"lx"] ) * ( lt1[,"lx"] / lt2[,"lx"] - dplyr::lead(lt1[,"lx"] / lt2[,"lx"]) )
    ex12_2[length(ex12_2)] = (lt1[nrow(lt1),"lx"] / lt1[1,"lx"]) * ( lt2[nrow(lt1),"Lx"] / lt2[nrow(lt1),"lx"] - lt1[nrow(lt1),"Lx"] / lt1[nrow(lt1),"lx"] )

    comp$ex12 <- round(0.5*(ex12_1 - ex12_2), 2)
    comp$ex12_prc = 100 * comp$ex12 / sum(comp$ex12)

  } else if(method == "pollard"){
    comp = data.frame(
      age = age,
      ex1 = lt1[, "ex"],
      ex2 = lt2[, "ex"]
    )
    comp$wx <- 0.5 * ( cumprod(1 - lt2[,"qx"]) * lt1[,"ex"] + cumprod(1 - lt1[,"qx"]) * lt2[,"ex"])
    Qx1 <- -log(dplyr::lead(lt1[,"lx"]) / lt1[,"lx"])
    Qx2 <- -log(dplyr::lead(lt2[,"lx"]) / lt2[,"lx"])
    Qx1[length(Qx1)] = Qx2[length(Qx1)] = 0
    comp$Qx1 = Qx1
    comp$Qx2 = Qx2
    comp$ex12 = round( (Qx1 - Qx2) * comp$wx, 2)
    comp$ex12_prc = 100 * comp$ex12 / sum(comp$ex12)
  }

  return(comp)
}

#' Split continuous ages to age groups
#'
#' @param x Numeric array. Values of age that should be grouped
#' @param groups Numeric array. Values of lower boundaries of age groups
#' @param char Logical. Should output be numeric (`FALSE`) or character (`TRUE`)? By default, `FALSE`
#' @param below_min_val What value to return for x < min(groups) when char = `FALSE`? Default is NA.
#' @return If `char = TRUE`, a factor of the same length as `x` with age-group
#'   labels. Otherwise, a numeric vector of the same length as `x` with the
#'   lower boundaries from `groups`; values below the first group receive
#'   `below_min_val`.
#' @examples
#' groups <- c(0, 1, 5, 10, 15, 20, 24, 45, 85)
#' x <- 0:100
#' ages(x, groups, TRUE)
#'
#' @export
ages <- function(x, groups, char = FALSE, below_min_val = NA) {

  breaks_full <- c(-Inf, groups, Inf)

  res_cut <- cut(x, breaks = breaks_full, right = FALSE)

  if(char){
    lab_below <- paste0("<", groups[1])

    starts <- groups[-length(groups)]
    ends <- groups[-1] - 1 #so it works only with integers

    lab_mids <- paste0(starts, "-", ends)
    lab_mids <- ifelse(lab_mids == "0-0", "0-1", lab_mids)

    lab_last <- paste0(groups[length(groups)], "+")

    all_labels <- c(lab_below, lab_mids, lab_last)
    levels(res_cut) <- all_labels

    return(res_cut)

  }else{

    map_values <- c(below_min_val, groups)

    return(map_values[as.integer(res_cut)])

  }
}

test_that("ages function handles first interval correctly", {
  age <- 0:100
  groups <- seq(10, 90, 10)
  res <- ages(x = age, groups = groups, char = FALSE, below_min_val = NA)
  expect_equal(sum(is.na(res)), sum(age < groups[1]))
})

#' Plot for mdecomp function
#'
#' @param x A result of age and cause decomposition from [mdecomp()].
#' @param return.data Should the data be returned (`TRUE`) or the ggplot2 plot (default, `FALSE`).
#' @param ... Ignored.
#'
#' @return If `return.data = FALSE`, a `ggplot2` object with stacked bars of
#'   age- and cause-specific contributions. If `return.data = TRUE`, a data
#'   frame with columns `age`, `ex12`, and `group` used to build the plot.
#' @seealso [mdecomp()]
#' @examples
#' data(mdecompex)
#' usa <- subset(mdecompex, cnt == "usa")
#' eng <- subset(mdecompex, cnt == "eng")
#' dec <- mdecomp(
#'   mx1 = list(
#'     all = usa$all,
#'     neoplasms = usa$neoplasms,
#'     circulatory = usa$circulatory
#'   ),
#'   mx2 = list(
#'     all = eng$all,
#'     neoplasms = eng$neoplasms,
#'     circulatory = eng$circulatory
#'   ),
#'   age = usa$age
#' )
#' plot(dec)
#' @method plot mdecomp
#' @export
plot.mdecomp <- function(x, return.data = FALSE, ...){
  decm <- x
  decm_plot <- decm[,c(1,3)]
  decm_plot$group = colnames(decm)[3]
  colnames(decm_plot)[2]<-"ex12"
  for(i in 4:ncol(decm)){
    decm_plot_i <- decm[,c(1,i)]
    decm_plot_i$group = colnames(decm)[i]
    colnames(decm_plot_i)[2]<-"ex12"
    decm_plot <- rbind(decm_plot,decm_plot_i)
    rm(decm_plot_i)
  }
  for (i in unique(decm_plot$group)){
    decm_plot[decm_plot$group==i,]$group <- paste0(i, " (", round(sum(decm_plot[decm_plot$group==i,]$ex12),2), ")")
  }
  finplot <- decm_plot %>%
    group_by(group) %>%
    mutate(age = ages(x = age, groups =  age, char = T)) %>%
    as.data.frame()
  if(return.data){
    return(finplot)
  }else{
    finplot <- finplot %>%
      ggplot(aes(x = age, y = ex12, fill = group))+
      geom_bar(stat="identity", colour = "black")
    return(finplot)
  }
  return(finplot)
}

#' Total Fertility Rate (TFR)
#'
#' @param fx Numeric array of age specific fertility rates.
#' @param age.int Numeric. Age group: `1` for one-year, `5` for five-year. Any age groups are allowed.
#'
#' @return A length-1 numeric value equal to the sum of age-specific fertility
#'   rates multiplied by `age.int`.
#' @examples
#' fx <- c(0.02, 0.08, 0.12, 0.09, 0.04, 0.01, 0.001)
#' tfr(fx, age.int = 5)
#' @export
tfr <- function(fx, age.int = 1){
  fx <- as.numeric(c(fx))
  return(sum(fx)*age.int)
}

#' Years of Life Lost (YLL) calculation
#'
#' @param Dx Array with the number of deaths.
#' @param type Character. Type of YLL to calculate. See details section.
#' @param age.int Numeric. Age interval of Dx. Can be `1` or `5`.
#' @param Dx_all Array with the number of all deaths. Used only with `yll.p` type, where `Dx` is array with cause-specific deaths.
#' @param pop Array with population. Used only with `yll.r` and `asyr` types.
#' @param w Array with population weights for direct standardization. Used only with `asyr` type.
#' @param standard Data frame. User-specific standard life expectancy to calculate YLL with the following columns: age, ex. Note: the `age.int` argument should be consistent with the `age` column in this data frame.
#' @details
#' Computes four types of Years of Life Lost (YLL) indicators:
#' \itemize{
#'   \item \strong{Absolute YLL} (\code{type = "yll"}):
#'     \deqn{YLL_{x,t,c} = D_{x,t,c} \times SLE_x}
#'     where \eqn{x,t,c} are age, time, and cause respectively, \eqn{D_{x,t,c}} is deaths in age \eqn{x} at time \eqn{t} from cause \eqn{c} and \eqn{SLE_x} is standard life expectancy at age \eqn{x}
#'   \item \strong{YLL proportion} (\code{type = "yll.p"}):
#'     \deqn{YLL_{x,t,c}^p = \frac{YLL_{x,t,c}}{YLL_{x,t}}}
#'     where \eqn{YLL_{x,t} = \sum_{c} YLL_{x,t,c}} (total YLL across causes)
#'
#'   \item \strong{YLL rate} (\code{type = "yll.r"}):
#'     \deqn{YLL_{x,t,c}^r = \left( \frac{YLL_{x,t,c}}{N_{x,t}} \right) \times 100'000}
#'     where \eqn{N_{x,t}} is population in age group \eqn{x} at time \eqn{t}
#'
#'   \item \strong{Age-standardized YLL rate} (\code{type = "asyr"}):
#'     \deqn{ASYR_{t,c} = \sum_{x=\alpha}^{\omega} \left( YLL_{x,t,c}^r \times w_x \right)}
#'     where \eqn{\alpha} to \eqn{\omega} corresponds to first to last age group, \eqn{w_x} is standard population weight for age \eqn{x}.
#' }
#' @references
#' Martinez, R., Soliz, P., Caixeta, R., & Ordunez, P. (2019). Reflection on modern methods: years of life lost due to premature mortality—a versatile and comprehensive measure for monitoring non-communicable disease mortality. *International Journal of Epidemiology*, *48*, 1367–1376.
#' @return A list whose components depend on `type`. For `type = "yll"`, the
#'   list contains `yll_all` (overall YLL) and `yll` (age-specific YLL). For
#'   `type = "yll.p"`, it contains `yll.p_all` and `yll.p` (overall and
#'   age-specific YLL proportions). For `type = "yll.r"`, it contains
#'   `yll.r_all` and `yll.r` (overall and age-specific YLL rates). For
#'   `type = "asyr"`, it contains `asyr_all` and `asyr` (overall and
#'   age-specific age-standardized YLL rates).
#' @examples
#' yll(Dx = rep(1, 19), type = "yll", age.int = 5)$yll_all
#' @export
#'
yll <- function(Dx, type = c("yll", "yll.p", "yll.r", "asyr"), age.int = 5, Dx_all = NULL, pop = NULL, w = NULL, standard = NULL){

  `%notin%` <- Negate(`%in%`)

  ndx = length(Dx)
  ndx_o = ndx
  if(is.null(standard)){
    sle <- demor::sle_stand
  } else {
    sle <- standard %>% mutate(stand = 1) %>% arrange(age)
  }


  if(age.int %notin% c(1,5)){
    stop("Age interval (age.int) should be 1 or 5")
  }
  if(type %notin% c("yll", "yll.p", "yll.r", "asyr")){
    stop("type should be one of the following: 'yll', 'yll.p', 'yll.r', 'asyr'")
  }

  if(age.int == 5){
    sle = sle[sle$stand==5,]$ex
    if(ndx != 19){
      excess = sum(Dx[length(sle):ndx])
      Dx <- Dx[1:length(sle)]
      ndx = length(Dx)
      Dx[ndx] = excess
    }
  }else{
    sle = sle[sle$stand==1,]$ex
    if(ndx > length(sle)){
      excess = sum(Dx[length(sle):ndx])
      Dx <- Dx[1:length(sle)]
      ndx = length(Dx)
      Dx[ndx] <- excess
    }
  }

  if(type == "yll"){

    return(
      list(
        yll_all = sum(sle[1:ndx] * Dx),
        yll = sle[1:ndx] * Dx
      )
    )

  } else if (type == "yll.p"){

    if(is.null(Dx_all)){
      stop("Cannot find Dx_all")
    }
    if(ndx_o != length(Dx_all)){
      stop("Lengths of Dx and Dx_all are different")
    }

    ndx_a <- length(Dx_all)
    if(ndx_a > length(sle)){
      excess = sum(Dx_all[length(sle):ndx_a])
      Dx_all <- Dx_all[1:length(sle)]
      ndx_a = length(Dx_all)
      Dx_all[ndx_a] <- excess
    }

    return(
      list(
        yll.p_all = ( sum(sle[1:ndx] * Dx) ) / sum(sle[1:ndx_a] * Dx_all),
        yll.p = ( sle[1:ndx] * Dx ) / ( sle[1:ndx_a] * Dx_all )
      )
    )

  } else if (type == "yll.r"){

    if(is.null(pop)){
      stop("Cannot find pop")
    }
    if(ndx_o != length(pop)){
      stop("Lengths of Dx and pop are different")
    }

    npx <- length(pop)
    if(npx > length(sle)){
      excess = sum(pop[length(sle):npx])
      pop <- pop[1:length(sle)]
      npx = length(pop)
      pop[npx] <- excess
    }

    return(
      list(
        yll.r_all = 100000 * sum(sle[1:ndx] * Dx)/sum(pop),
        yll.r = 100000 * (sle[1:ndx] * Dx) / pop
      )
    )

  }else if (type == "asyr"){

    if(is.null(pop)){
      stop("Cannot find pop")
    }
    if(ndx_o != length(pop)){
      stop("Lengths of Dx and pop are different")
    }
    if(length(pop) != length(w)){
      stop("Lengths of w and pop are different")
    }

    npx <- length(pop)
    if(npx > length(sle)){
      excess = sum(pop[length(sle):npx])
      pop <- pop[1:length(sle)]
      npx = length(pop)
      pop[npx] <- excess
    }

    nwx <- length(w)
    if(nwx > length(sle)){
      excess = sum(w[length(sle):nwx])
      w <- w[1:length(sle)]
      nwx = length(w)
      w[nwx] <- excess
    }

    return(
      list(
        asyr_all = sum( w * (100000 * (sle[1:ndx] * Dx) / pop) ),
        asyr =  w * (100000 * (sle[1:ndx] * Dx) / pop)
      )
    )
  }
}

#' The Human Life Indicator (HLI)
#'
#' @param age Numeric array of age intervals; for full life table = `0:100`; for concise life table = `c(0:1, seq(5,85,5))`
#' @param mx Numeric array with age specific mortality rates.
#' @param ... Optional. Additional arguments for [LT()] function.
#' @references Ghislandi, S., Sanderson, W.C., & Scherbov, S. (2019). A Simple Measure of Human Development: The Human Life Indicator. *Population and Development Review*, *45*, 219–233.
#' @details
#' It is calculated as
#' \deqn{HLI = \prod_{x=\alpha}^{\omega}(x + a_x)^{d_x}}
#' where \eqn{\alpha, \omega} are the first and last age groups, \eqn{x} is age, \eqn{a_x, d_x} are life table functions (s.t. \eqn{\sum_{x=\alpha}^{\omega} d_x = 1}).
#' @return A length-1 numeric value giving the Human Life Indicator, i.e. the
#'   geometric mean age at death implied by the life table.
#' @examples
#' age <- 0:5
#' mx <- c(0.02, 0.01, 0.012, 0.015, 0.02, 0.03)
#' hli(age, mx)
#' @export
hli <- function(age, mx, ...){
  lt <- LT(age = age, mx = mx, ...)
  if(round(sum(lt[,"dx"]), 2) > 1){
    lt[,"dx"] <- lt[,"dx"] / sum(lt[,"dx"])
  }
  val <- exp(sum(log(lt[,"age"] + lt[,"ax"])*lt[,"dx"]))
  return(val)
}

test_that("hli works on abridged and full life tables", {
  age_full <- 0:100
  age_abr  <- c(0:1, seq(5, 85, 5))
  mx_full  <- rep(0.01, length(age_full))
  mx_abr   <- rep(0.01, length(age_abr))

  h_full <- hli(age_full, mx_full)
  h_abr  <- hli(age_abr,  mx_abr)

  expect_true(is.finite(h_full) && h_full > 0)
  expect_true(is.finite(h_abr)  && h_abr  > 0)

  # They won't be identical, but should be in the same ballpark.
  expect_lt(abs(log(h_full / h_abr)), 0.1)  # ~10% tolerance; adjust if needed
})

#' Mean Age at Childbearing (MAC)
#'
#' @param fx Numeric array of age specific fertility rates.
#' @param age Numeric array of ages. For example, `15:55` for 1-year age-groups
#'
#' @return A length-1 numeric value giving the mean age at childbearing implied
#'   by `fx`.
#' @examples
#' age <- seq(15, 45, 5)
#' fx <- c(0.03, 0.10, 0.14, 0.12, 0.07, 0.03, 0.01)
#' mac(fx, age)
#' @export
mac <- function(fx, age){
  fx <- as.numeric(c(fx))
  delta <- (diff(age)[1] * fx)/tfr(fx, diff(age)[1])
  fin = sum( (age + 0.5*diff(age)[1])*delta )
  return(round(fin, 2))
}

#' Associated single decrement life table (ASDT) for causes of death (cause-deleted life table)
#'
#' @param age Numeric array of age intervals; for full life table = `0:100`; for concise life table = `c(0:1, seq(5,85,5))`
#' @param m_all Numeric array with age specific mortality rates of all causes of death (usual mx).
#' @param m_i Numeric array with age specific mortality rates of some cause of death (i)
#' @param full Logical. Is full table needed? `TRUE` for full, `FALSE` for concise. By default, `FALSE`
#' @param method Character. The method of ASDT construction to use. Now just "chiang1968" is supported.
#' @param ... Optional. Additional arguments for [LT()] function.
#' @references Chiang, L. (1968). *Introduction to Stochastic Processes in Biostatistics*. New York: John Wiley and Sons.
#' @return If `full = TRUE`, a data frame with the full associated single
#'   decrement life table, including the standard life-table columns and the
#'   cause-deleted columns `r_not_i`, `p_not_i`, `l_not_i`, `a_not_i`,
#'   `d_not_i`, `L_not_i`, `T_not_i`, and `ex_without_i`. If `full = FALSE`, a
#'   reduced data frame with columns `age`, `r_not_i`, `lx`, `qx`, `ax`, `ex`,
#'   `p_not_i`, `l_not_i`, `a_not_i`, and `ex_without_i`.
#' @examples
#' data(asdtex)
#' asdt(
#'   age = asdtex$age,
#'   m_all = asdtex$all,
#'   m_i = asdtex$circulatory
#' )[1:3, ]
#' @export
asdt <- function(age, m_all, m_i, full = FALSE, method = "chiang1968", ...){
  lt <- as.data.frame(LT(age = age, mx = m_all, ...))
  lt$r_not_i <- 1-m_i/m_all
  lt$p_not_i <- (1-lt$qx)^lt$r_not_i
  lt$l_not_i <- 1
  for (i in 2:nrow(lt)){
    lt[i,]$l_not_i <- lt[i-1,]$l_not_i*lt[i-1,]$p_not_i
  }
  lt$n <- c(diff(lt$age,1),NA)
  lt$a_not_i <- NA
  for(i in 1:nrow(lt)){
    if(i==nrow(lt)){
      lt[i,]$a_not_i <- lt[i,]$ex/lt[i,]$r_not_i
    } else {
      lt[i,]$a_not_i <- lt[i,]$n + lt[i,]$r_not_i * (lt[i,]$qx/(1-lt[i,]$p_not_i))*(lt[i,]$ax - lt[i,]$n )
    }
  }
  lt[i,]$a_not_i <- round(lt[i,]$a_not_i,3)
  lt$d_not_i <- lt$l_not_i * (1-lt$p_not_i)
  lt$L_not_i <- lt$n * lt$l_not_i - lt$n * (1 - lt$a_not_i) * lt$d_not_i
  lt[nrow(lt),]$L_not_i <- lt[nrow(lt),]$l_not_i * lt[nrow(lt),]$a_not_i
  lt$T_not_i <- rev(cumsum(rev(lt$L_not_i)))
  lt$ex_without_i <- round(lt$T_not_i / lt$l_not_i, 2)
  if (full == T){
    return(lt)
  }else{
    return(lt[,c("age", "r_not_i", "lx", "qx", "ax", "ex", "p_not_i", "l_not_i", "a_not_i", "ex_without_i")])
  }
}

#' Multiple Decrement Life Table
#'
#' @param age Numeric array of age intervals; for full life table = `0:100`; for concise life table = `c(0:1, seq(5,85,5))`
#' @param mx List of numeric arrays. 1st array should be all-cause mx in the population, other arrays are cause-specific mx in the population.
#' @param ... Other parameters for the function [LT()] including `ax` (by default, the middle of the interval), `sex` (by default = `"m"` - males), `l0` (by default = 1).
#' @seealso [LT()] for usual life table calculation
#' @return A numeric matrix extending the output of [LT()]. In addition to the
#'   standard life-table columns, for each cause `i` in `mx[-1]` it adds
#'   `qx_i`, `dx_i`, `lx_i`, and `ex_no_i`, corresponding to cause-specific
#'   death probabilities, deaths, survivors, and cause-deleted life expectancy.
#' @examples
#' data(asdtex)
#' mx_causes <- list(
#'   all = asdtex$all,
#'   neoplasms = asdtex$neoplasms,
#'   circulatory = asdtex$circulatory
#' )
#' MLT(age = asdtex$age, mx = mx_causes)[1:3, c("age", "mx", "qx_neoplasms", "ex_no_neoplasms")]
#' @export
MLT <- function(age, mx, ...){
  if(length(mx)<2){
    stop("In the list should be at least 2 arrays (1 - mx, 2 - mx_i, ...)")
  }
  lt <- LT(age = age, mx = mx[[1]], ...)
  for(i in names(mx)[-1]){
    qx_i = round(lt[,"qx"]*mx[[i]]/lt[,"mx"],5)
    lt <- cbind(lt, qx_i)
    colnames(lt)[ncol(lt)] <- paste0("qx_", i)
    dx_i = round(lt[,paste0("qx_", i)]*lt[,"lx"],5)
    lt <- cbind(lt, dx_i)
    colnames(lt)[ncol(lt)] <- paste0("dx_", i)
    lx_i = round(rev(cumsum(rev(lt[,paste0("dx_", i)]))),5)
    lt <- cbind(lt, lx_i)
    colnames(lt)[ncol(lt)] <- paste0("lx_", i)
    ex_i = asdt(age = age, m_all = mx[[1]], m_i = mx[[i]], full = FALSE, ...)[,"ex_without_i"]
    lt <- cbind(lt, ex_i)
    colnames(lt)[ncol(lt)] <- paste0("ex_no_", i)
  }
  return(lt)
}

#' Age and cause decomposition of differences in life expectancies
#'
#' @param mx1 List of numeric arrays. 1st array should be all-cause nmx in the 1st population, other arrays are cause-specific nmx in the 1st population
#' @param mx2 List of numeric arrays. 1st array should be all-cause nmx in the 2nd population, other arrays are cause-specific nmx in the 2nd population
#' @param age Numeric array of age intervals; for full life table = `0:100`; for concise life table = `c(0:1, seq(5,85,5))`
#' @param method Character. Decomposition method. "andreev" (1982) or "arriaga" (1984) - slightly different in their results. By default, `method = "andreev"`.
#' @param ... Optional. Additional arguments for [decomp()].
#' @details
#' The contribution of each cause \eqn{c} to the absolute difference in life expectancies between the first and second population is caculated as
#' \deqn{\Delta_{x,c} = \frac{m^1_{x,c} - m^2_{x,c}}{m^1_{x} - m^2_{x}} \times \Delta_{x}}
#' where \eqn{\Delta_{x}} is contribution of age \eqn{x} to difference \eqn{e_0^2 - e_0^1} from function [decomp()], \eqn{m^i_{x,c}} is age-specific mortality rate for population \eqn{i} from cause \eqn{c}, and \eqn{m^i_x} is total age-specific mortality rate.
#' @seealso [decomp()] for just age decomposition and [plot.mdecomp()] for graph of `mdecomp` results
#' @return A data frame of class `c("mdecomp", "data.frame")` with one row per
#'   age group. Column `age` contains ages, `ex12` contains the overall age
#'   contribution to the life-expectancy difference, and the remaining columns
#'   contain cause-specific contributions named after the cause-specific
#'   elements of `mx1` and `mx2`.
#' @examples
#' data(mdecompex)
#' usa <- subset(mdecompex, cnt == "usa")
#' eng <- subset(mdecompex, cnt == "eng")
#' dec <- mdecomp(
#'   mx1 = list(
#'     all = usa$all,
#'     neoplasms = usa$neoplasms,
#'     circulatory = usa$circulatory
#'   ),
#'   mx2 = list(
#'     all = eng$all,
#'     neoplasms = eng$neoplasms,
#'     circulatory = eng$circulatory
#'   ),
#'   age = usa$age
#' )
#' dec[1:3, ]
#' @export
mdecomp <- function(mx1, mx2, age, method = "andreev", ...){
  if (length(mx1)!=length(mx2)){
    stop("lengths (number of causes) of mx1 and mx2 are not equal")
  }
  if (length(mx1[[1]])!=length(mx2[[1]]) | length(mx1[[1]])!=length(age)){
    stop("lengths (number of age-groups) of mx1 and mx2 are not equal")
  }
  ex12 <- decomp(mx1[[1]], mx2[[1]], age = age, method = method, ...)$ex12
  mdec <- data.frame(age = age, ex12 = ex12)
  for (i in 2:length(mx1)){
    mdec[paste0(names(mx1)[i])] <- NA
    mdec[paste0(names(mx1)[i])] <- ex12 * (mx1[[i]] - mx2[[i]]) / (mx1[[1]] - mx2[[1]])
  }
  class(mdec) <- c("mdecomp", "data.frame")
  return(mdec)
}

#' Gini coefficient of a life table
#'
#' @param age Numeric array of age intervals
#' @param mx Numeric array with age specific mortality rates.
#' @param ... Optional. Additional arguments for [LT()] function.
#' @return A list with two components: `Gini`, itself a list with the relative
#'   Gini coefficient `G0` and the absolute Gini coefficient `G0_abs`; and
#'   `plot`, a data frame with columns `Fx` and `Phix` for drawing the Lorenz
#'   curve.
#' @examples
#' age <- 0:5
#' mx <- c(0.02, 0.01, 0.012, 0.015, 0.02, 0.03)
#' gini(age, mx)$Gini
#' @export
gini <- function(age, mx, ...){
  lt <- LT(age = age, mx = mx, ...)
  Fx = rep(0, nrow(lt))
  Phix = rep(0, nrow(lt))
  for(i in 1:(nrow(lt) - 1)){
    Fx[i] = 1 - lt[i+1,"lx"]/lt[1,"lx"]
    Phix[i] = ( lt[1,"Tx"] - (lt[i,"Tx"] + lt[i,"age"]*lt[i,"lx"]) )/lt[1,"Tx"]
  }
  Fx[nrow(lt)] = Phix[nrow(lt)] = 1
  Fx[1] = Phix[1] = 0
  G0 = 0
  for(i in  1:(nrow(lt))){
    for(j in 1:(nrow(lt))){
      G0 = G0 + (1/(2*lt[1,"ex"]*lt[1,"lx"]^2)) *
        (lt[i,"dx"] * lt[j,"dx"]) *
        abs(lt[i,"age"] + lt[i,"ax"] - lt[j,"age"] - lt[j,"ax"])
    }
  }
  G0_abs = G0 * lt[1,"ex"]
  return(list(Gini = list(G0 = G0, G0_abs = G0_abs), plot = data.frame(Fx, Phix)))
}

#' Data on mortality of US men in 2002 by some causes
#'
#'
#' @format ## A dataframe
#' \describe{
#'   \item{age}{age}
#'   \item{neoplasms}{age specific mortality rate (asmr) from Neoplasms}
#'   \item{circulatory}{asmr from Diseases of the circulatory system}
#'   \item{respiratory}{asmr from Diseases of the respiratory system}
#'   \item{digestive}{asmr from Diseases of the digestive system}
#'   \item{other}{asmr from other causes}
#'   \item{all}{overall asmr}
#' }
#' @source Andreev & Shkolnikov spreadsheet, can be seen on this [webpage](https://www.demogr.mpg.de/en/publications_databases_6118/publications_1904/mpidr_technical_reports/an_excel_spreadsheet_for_the_decomposition_of_a_difference_between_two_values_of_an_aggregate_demographic_4591)
"asdtex"

#' Data on mortality of US and England and Wales men in 2002 by some causes
#'
#'
#' @format ## A dataframe
#' \describe{
#'   \item{age}{age}
#'   \item{neoplasms}{age specific mortality rate (asmr) from Neoplasms}
#'   \item{circulatory}{asmr from Diseases of the circulatory system}
#'   \item{respiratory}{asmr from Diseases of the respiratory system}
#'   \item{digestive}{asmr from Diseases of the digestive system}
#'   \item{other}{asmr from other causes}
#'   \item{all}{overall asmr}
#'   \item{cnt}{country: `usa` - US, `eng` - England and Wales}
#' }
#' @source Andreev & Shkolnikov spreadsheet, can be seen on this [webpage](https://www.demogr.mpg.de/en/publications_databases_6118/publications_1904/mpidr_technical_reports/an_excel_spreadsheet_for_the_decomposition_of_a_difference_between_two_values_of_an_aggregate_demographic_4591)
"mdecompex"

#' Data on Standard Life expectancies that is used for YLL calculations
#'
#'
#' @format ## A dataframe
#' \describe{
#'   \item{stand}{Standard: 1 - World Health Organization Standard Life Expectancy by single-age; 2 - Global Burden of Disease studies (GBD) and WHO Global Health Estimates (WHO GHE) Standard Life Expectancy by 5-year age groups.}
#'   \item{age}{age}
#'   \item{ex}{Standard Life Expectancy}
#' }
#' @source Martinez, R., Soliz, P., Caixeta, R., Ordunez, P. (2019). Reflection on modern methods: years of life lost due to premature mortality-a versatile and comprehensive measure for monitoring non-communicable disease mortality. International Journal of Epidemiology, 48, 1367-1376. \doi{10.1093/ije/dyy254}
"sle_stand"

#' Rosbris' Region codes
#'
#'
#' @format ## A dataframe
#' \describe{
#'   \item{n}{number of Region (sequence number)}
#'   \item{name}{Region name}
#'   \item{code}{Region unique code}
#' }
#' @source Russian Fertility and Mortality Database. Center for Demographic Research, Moscow (Russia). Available at https://www.nes.ru/demogr-fermort-data
"rosbris.codes"

#' e-dagger
#'
#' @param age Numeric array of age intervals
#' @param mx Numeric array with age specific mortality rates.
#' @param ... Optional. Additional arguments for [LT()] function.
#'
#' @return A named numeric vector of the same length as `age`, where each
#'   element is `e^\dagger_x`, the average remaining life years lost because of
#'   death from age `x` onward.
#' @examples
#' age <- 0:5
#' mx <- c(0.02, 0.01, 0.012, 0.015, 0.02, 0.03)
#' edagger(age, mx)[1:3]
#' @export
edagger <- function(age, mx, ...){
  lt <- LT(age = age, mx = mx, ...)
  ed = rep(0, length(age))
  for(i in 1:(length(ed) - 1)){
    ed[i] <- (1/lt[i,"lx"]) *
      sum( lt[i:(length(ed)-1),"dx"] *
             # ( lt[(i+1):(length(ed)), "ex"] + 1 - lt[i:(length(ed)-1),"ax"] )
             ( lt[(i+1):length(ed), "ex"] + lt[i:(length(ed)-1),"ex"] ) / 2
      ) +
      lt[length(ed),"dx"] / 2*lt[i,"lx"]
  }
  names(ed) <- age
  return(ed)
}

if (getRversion() >= "2.15.1") {
  utils::globalVariables(
    c(
      ".",
      "code",
      "fx",
      "fxA",
      "Group",
      "N",
      "parity",
      "Reg",
      "Sex",
      "age",
      "age2",
      "conf.high",
      "conf.low",
      "ex",
      "ex12",
      "group",
      "mx",
      "pop",
      "pop2",
      "sex",
      "territory",
      "Year",
      "year"
    )
  )
}

#' Plot population pyramid
#'
#' @param popm Numeric vector of male population
#' @param popf Numeric vector of female population
#' @param popm2 Optional. Numeric vector of second male population to be plotted as line.
#' @param popf2 Optional. Numeric vector of second female population to be plotted as line.
#' @param age Numeric vector of ages
#' @param prc Should the population be shown as a percentage rather than in absolute terms? By default, it is `FALSE` and the plot shows user-specific values in `popm` and `popf`.
#' @param sexn Character array of labels for the sexes. By default = `c("Males", "Females")`
#' @param sexc Character array of colors for the sexes. By default = `c("#ED0000B2","#00468BB2")`, which are red and blue
#' @param age.cont Optional. Logical. Should the age axis be considered as continuous scale? Recommend to switch to `TRUE` if the age intervals are small (1 year) and `FALSE` otherwise (for ex., when age interval is 5 years). By default function chooses the value by itself.
#' @param un.intervals Logical. Should the age groups be unified? For example, if the first and second groups are 0-1, 1-4, and all other groups are five-year, the function automatically makes the group 1-4 to make all the intervals the same. By default, `TRUE`.
#'
#' @return A `ggplot2` object representing a population pyramid. If `popm2` and
#'   `popf2` are supplied, the plot also includes dashed comparison lines for
#'   the second population.
#' @examples
#' plot_pyr(
#'   popm = c(100, 90, 80, 70, 60),
#'   popf = c(95, 92, 85, 75, 65),
#'   age = seq(0, 20, 5)
#' )
#' @import ggplot2 scales
#' @export
plot_pyr <- function(popm, popf, popm2 = NULL, popf2 = NULL, age, prc = FALSE, sexn = c("Males", "Females"), sexc = c("#ED0000B2","#00468BB2"), age.cont = NULL, un.intervals = TRUE){

  if (length(popm) != length(popf) || length(popm) != length(age)) {
    stop("popm, popf, and age must have the same length.")
  }
  if(is.null(age.cont)){
    age.cont <- if (length(age) > 50) TRUE else FALSE
  }
  if(is.null(popm2)){
    popm2 <- popf2 <- rep(0, length(age))
  }
  pyr = data.frame(age = rep(age,2),
                   pop = c(popm, popf),
                   sex = c(rep(sexn[1], length(age)), rep(sexn[2], length(age))),
                   pop2 = c(popm2, popf2)
  )

  if(prc){
    pyr <- pyr %>%
      group_by(sex) %>%
      mutate(pop = pop / sum(pop),
             pop2 = pop2 / sum(pop2)
      ) %>%
      as.data.frame()
    pyr[is.na(pyr)] <- 0
  }

  if(un.intervals){
    if(length(unique(diff(age))) > 1){
      ux <- unique(diff(age))
      agegroup <- ux[which.max(tabulate(match(diff(age), ux)))]
      newage <- seq(0, max(age), agegroup)
      newage0 <- newage
      newage0[1] = newage0[2] - 1
      pyr$age <- demor::ages(x = age, groups = newage0, char = FALSE)
      pyr[which(pyr[,"age"] == newage0[1]), "age"] <- newage[1]
      pyr <- stats::aggregate(data = pyr, cbind(pop, pop2) ~ sex + age, FUN = sum)
    }
  }

  pyr$ex <- NA
  pyr[pyr$sex == sexn[1],]$ex <- ifelse(pyr[pyr$sex == sexn[1],]$pop - pyr[pyr$sex == sexn[2],]$pop > 0, pyr[pyr$sex == sexn[2],]$pop - pyr[pyr$sex == sexn[1],]$pop, 0)
  pyr[pyr$sex == sexn[2],]$ex <- ifelse(pyr[pyr$sex == sexn[1],]$pop - pyr[pyr$sex == sexn[2],]$pop < 0, abs(pyr[pyr$sex == sexn[1],]$pop - pyr[pyr$sex == sexn[2],]$pop), 0)

  pyr$pop = ifelse(pyr$sex==sexn[1], -1*pyr$pop, pyr$pop)
  pyr$pop2 = ifelse(pyr$sex==sexn[1], -1*pyr$pop2, pyr$pop2)
  if(age.cont){
    pl <- ggplot(pyr %>% arrange(sex, age), aes(age, pop, fill = sex))+
      geom_bar(stat = "identity", alpha = 0.25, color = "black", linewidth = 0.1)+
      geom_bar(aes(y = pop - ex, fill = sex), stat = "identity", color = "black", alpha = 1, linewidth = 0.1)+
      coord_flip()+
      scale_x_continuous(breaks = seq(min(age), max(age), 5))+
      scale_y_continuous(sec.axis = dup_axis(),
                         breaks = scales::breaks_extended(n = 10),
                         labels = scales::label_number(scale_cut = scales::cut_short_scale(), style_negative = "parens")
      )+
      labs(x = "", y = "")+
      theme_linedraw()+
      theme(legend.position = "bottom")+
      geom_line(aes(x = age, y = pop2), linetype = "dashed")+
      scale_fill_manual(name = "", values = sexc)
  }else{
    pl <- ggplot(pyr %>% arrange(sex, age) %>% mutate(age = as.factor(age)) %>% mutate(age2 = as.numeric(age)), aes(age, pop, fill = sex))+
      geom_bar(stat = "identity", alpha = 0.25, color = "black", linewidth = 0.1, width = 1)+
      geom_bar(aes(y = pop - ex, fill = sex), stat = "identity", color = "black", alpha = 1, linewidth = 0.1, width = 1)+
      coord_flip()+
      scale_y_continuous(sec.axis = dup_axis(),
                         breaks = scales::breaks_extended(n = 10),
                         labels = scales::label_number(scale_cut = scales::cut_short_scale(), style_negative = "parens")
      )+
      labs(x = "", y = "")+
      theme_linedraw()+
      theme(legend.position = "bottom")+
      geom_line(aes(x = age2, y = pop2), linetype = "dashed")+
      scale_fill_manual(name = "", values = sexc)
  }

  return(pl)
}

#' Tempo-adjusted total fertility rate (TFR')
#'
#' @param past_fx List with numeric arrays of age specific fertility rates for period t-1 by parity
#' @param present_fx List with numeric arrays of age specific fertility rates for period t by parity (it is period of interest)
#' @param post_fx List with numeric arrays of age specific fertility rates for period t+1 by parity
#' @param age Array with numeric values age
#' @details
#' This indicator is calculated as follows
#' \deqn{TFR_{i,t}' = \frac{TFR_{i,t}}{1-(M_{i,t+1} - M_{i,t-1}) / 2}}
#' where \eqn{TFR_{i,t}', TFR_{i,t}} are tempo-adjusted and usual total fertility rate for parity \eqn{i} and time \eqn{t} respectively, \eqn{M_{i,t}} is mean age at childbearing for parity \eqn{i} and time \eqn{t}. The tempo-adjusted total fertility rate is a sum of parity-specific \eqn{TFR_i'}.
#'
#' Note, the calculation are done as in footnote 1 in (Bongaarts & Feeney, 2000, p. 563). Unfortunately, the original 1998 article does not provide the exact formula, which has caused some confusion in academic circles.
#'
#' @seealso [tfr()] for TFR and [mac()] for mean age at childbearing calculation.
#'
#' @references
#' Bongaarts, J., & Feeney, G. (1998). On the Quantum and Tempo of Fertility. Population and Development Review, 24(2), 271–291. \doi{10.2307/2807974}
#'
#' Bongaarts, J., & Feeney, G. (2000). On the Quantum and Tempo of Fertility: Reply. Population and Development Review, 26(3), 560–564. \doi{10.1111/j.1728-4457.2000.00560.x}
#'
#' @return A list with four components: `tatfr` (overall tempo-adjusted total
#'   fertility rate), `tatfr_i` (parity-specific tempo-adjusted rates), `tfr`
#'   (overall conventional TFR), and `tfr_i` (parity-specific conventional
#'   rates).
#' @examples
#' age <- seq(15, 45, 5)
#' past_fx <- list(
#'   c(0.02, 0.05, 0.07, 0.05, 0.03, 0.01, 0.00),
#'   c(0.01, 0.03, 0.04, 0.03, 0.02, 0.01, 0.00)
#' )
#' present_fx <- list(
#'   c(0.03, 0.06, 0.08, 0.06, 0.03, 0.01, 0.00),
#'   c(0.01, 0.03, 0.05, 0.04, 0.02, 0.01, 0.00)
#' )
#' post_fx <- list(
#'   c(0.03, 0.05, 0.08, 0.07, 0.04, 0.02, 0.00),
#'   c(0.01, 0.03, 0.04, 0.04, 0.03, 0.01, 0.00)
#' )
#' tatfr(past_fx, present_fx, post_fx, age)
#' @export
tatfr <- function(past_fx, present_fx, post_fx, age){
  parity = length(past_fx)
  present_tfr = rt = tai = rep(NA, parity)

  for(i in 1:parity){
    present_tfr[i] <- demor::tfr(fx = present_fx[[i]], age.int = stats::median(diff(age)))

    rt[i] <- demor::mac(fx = post_fx[[i]], age = age) - demor::mac(fx = past_fx[[i]], age = age)
    rt[i] <- rt[i]/2

    tai[i] = present_tfr[i] / (1-rt[i])
  }

  return(list(tatfr = sum(tai), tatfr_i = tai, tfr = sum(present_tfr), tfr_i = present_tfr))
}

#' Life table
#'
#' @param age Numeric array of age intervals; for full life table = `0:100`; for concise life table = `c(0:1, seq(5,85,5))`
#' @param sex Character. Sex. `"m"` for males or `"f"` for females. By default = `"m"`.
#' @param mx Numeric array with age specific mortality rates.
#' @param ax Optional. Numeric array with ax. By default, it is the middle of the interval, while ax for age 0 is modeled as in Andreev & Kingkade (2015).
#' @param w Optional. Numeric array with weights for each age interval for calculating weighted life expectancy (`wex`).
#' @param l0 Numeric. Life table radix. By default, = `1` but it can be any positive real number. In "human" demography tradition it is 100'000, in "ecological" and "evolutionary" demography tradition it is 1.
#' @details
#' By default, \eqn{a_x} for age 0 (first entity in `ax`) is modeled as in Andreev & Kingkade (2015, p. 390, see table 3-2).
#'
#' The weighted life expectancy is calculated as follows:
#' \deqn{e_x^w = \frac{\sum_{i = x}^{\omega}L_x w_x}{l_x}}
#' where \eqn{\omega} is the last age, \eqn{w} is weight s.t. \eqn{w \in [0,1]}, and other variables are life table functions.
#'
#' @seealso [MLT()] for Multiple Decrement Life Table.
#' @references Andreev, E. M., & Kingkade, W. W. (2015). Average age at death in infancy and infant mortality level: Reconsidering the Coale-Demeny formulas at current levels of low mortality. *Demographic Research*, *33*, 363-390.
#' @return A numeric matrix with one row per age group. Standard columns are
#'   `age`, `mx`, `ax`, `qx`, `lx`, `dx`, `Lx`, `Tx`, and `ex`. If `w` is
#'   supplied, additional columns `w`, `wLx`, and `wex` are appended, where
#'   `wex` is weighted life expectancy.
#' @examples
#' # Minimal toy example
#' age <- 0:5
#' mx <- c(0.02, 0.01, 0.012, 0.015, 0.02, 0.03)
#' LT(age = age, sex = "m", mx = mx)
#' \donttest{
#' if (interactive()) {
#'   # Real RosBris data via get_rosbris(): Russian males, 2010
#'   rus2010 <- subset(get_rosbris("mortality_5"),
#'     year == 2010 & code == 1100 & sex == "m" & territory == "t"
#'   )
#'   LT(age = rus2010$age, sex = "m", mx = rus2010$mx)
#' }
#' }
#' @export
LT <- function(age, sex = "m", mx, ax = NULL, w = NULL, l0 = 1) {
  if(length(mx) != length(age)){
    stop("The length of user-specific array of age does not equal mx array's length!")
  }
  if(any(mx < 0)){
    mx[mx < 0] = 0
    warning("There are mx < 0, replace them with 0")
  }
  last <- length(mx)
  nx <- c(diff(age), 1)
  if(is.null(ax)){
    ax <- nx * 0.5
    if (sex == "m") {
      if (mx[1] < 0.02300) {
        ax[1] <- 0.14929 - (1.99545 * mx[1])
      }
      if ((0.0230 <= mx[1]) &
          (mx[1] < 0.08307)) {
        ax[1] <- 0.02832 + (3.26021 * mx[1])
      }
      if (0.08307 <= mx[1]) {
        ax[1] <- 0.29915
      }
    }
    if (sex == "f") {
      if (mx[1] < 0.01724) {
        ax[1] <- 0.14903 - (2.05527 * mx[1])
      }
      if ((0.01724 <= mx[1]) &
          (mx[1] < 0.06891)) {
        ax[1] <- 0.04667 + (3.88089 * mx[1])
      }
      if (0.06891 <= mx[1]) {
        ax[1] <- 0.31411
      }
    }
  }else{
    if(length(ax) != length(mx)){
      stop("The length of user-specific array of ax does not equal mx array's length!")
    }
  }

  if (mx[last] > 0) {
    ax[last] <- 1 / mx[last]
  }
  qx <- (nx * mx) / (1 + (nx - ax) * mx)
  qx[last] <- 1
  px <- 1 - qx
  lx <- l0
  for (i in 1:(last - 1)) {
    lx[i + 1] <- lx[i] * px[i]
  }
  dx <- lx * qx
  dx[last] <- lx[last]
  Lx = nx * lx - ax * dx
  Lx[last] = lx[last] * ax[last]
  Tx = rev(cumsum(rev(Lx)))
  ex <- Tx / lx
  lt <-
    as.matrix(
      data.frame (
        age,
        mx = round(mx, 5),
        ax = round(ax, 3),
        qx = round(qx, 5),
        lx = round(lx, 5),
        dx = round(dx, 5),
        Lx = round(Lx, 5),
        Tx = round(Tx, 5),
        ex = round(ex, 2)
      )
    )
  if(!is.null(w)){
    if(length(age) != length(w)){
      stop("The length of user-specific array of weights does not equal age array's length!")
    }else{
      wLx = Lx*w
      wTx <- rev(cumsum(rev(wLx)))
      wex <- round(wTx / lx, 2)
      lt <- cbind(lt, w, wLx = round(wLx, 5), wex)
    }
  }
  return(lt)
}

#' Lee-Carter model
#'
#' @param data Dataframe in the long format with the following columns: `age`, `year`, `mx` (age specific mortality rates). For some types of `ktadj` argument `N` (population at age x) and `Dx` (number of deaths at age x) columns should also be presented.
#' @param n Numeric. Forecasted horizon
#' @param alpha Numeric. The level of uncertainty. By default, `alpha = 0.05` for 95% CI.
#' @param model Character. Model type for kt forecasting. Can be "RWwD" for random walk with drift (by default, for original Lee-Carter model) or "ARIMA" for ARIMA model which parameters are chosen automatically by `forecast::auto.arima()`.
#' @param ax_method Character. Method for ax calculation. Can be "classic" from original Lee-Carter model (by default), "last" or "last_smooth". See details.
#' @param bx_method Character. Method for bx calculation. Can be "classic" from original Lee-Carter model (by default) and "rotate" for rotating bx (Li et al., 2013).
#' @param boot Logical. Should bootstrap estimates for uncertainty be used? `FALSE` by default.
#' @param bn Numeric. Used if `boot = TRUE`, number of bootstrap samples. By default, `bn = 1000`.
#' @param ktadj Character. Type of `kt` adjustment. It can be set to 'none' (defaukt, no adjustment), 'Dmin', 'e0min', 'poisson' or 'edaggermin' (see Details). Note that 'Dmin' and 'poisson' require data on the age-specific number of deaths (`Dx` column in the data) and the age-specific population (`N` column in the data).
#' @param ... Optional. Additional arguments for [LT()] function.
#' @details
#' The \strong{`model`} argument specifies the forecasting method.
#' \itemize{
#'   \item `model ="RWwD"` – classic random walk option
#'   \item `model = "ARIMA"` for selecting a more complex time series model
#' }
#'
#' The \strong{`ax_method`} argument allows to control how `a_x` is calculated.
#' \itemize{
#'   \item `ax_method = "classic"` – classic option with the average of the logarithm of mortality rates (but there is so-called "jump-off bias").
#'   \item `ax_method = "last"` uses the logarithm of mortality for the last available year (as proposed in Lee & Miller, 2001).
#'   \item `ax_method = "last_smooth"` uses data for the last year with smoothing (see Ševčíková et al., 2016, p. 288).
#' }
#'
#' The \strong{`bx_method`} argument allows to control how `b_x` is calculated.
#' \itemize{
#'   \item `bx_method = "classic"` for the original method.
#'   \item `bx_method = "rotate"` for the rotational variant (see Li et al., 2013).
#' }
#'
#' The \strong{`ktadj`} argument allows to control how `k_t` is calculated.
#' \itemize{
#'   \item `ktadj = "none"` for no adjustment.
#'   \item `ktadj = "Dmin"` for minimizing the deviance of predicted/actual annual deaths (as proposed in the original Lee-Carter paper). This method requires data on the age-specific number of deaths (`Dx` column in the data) and the age-specific population (`N` column in the data).
#'   \item `ktadj = "e0min"` for minimizing the deviance of predicted/actual life expectancy (as proposed in Lee & Miller, 2001).
#'   \item `ktadj = "poisson"` for minimizing the deviance from a Poisson model, where the dependent variable is the age-specific annual number of deaths (as proposed in Booth et al., 2002). This method requires data on the age-specific number of deaths (`Dx` column in the data) and the age-specific population (`N` column in the data).
#'   \item `ktadj = "edaggermin"` for minimizing the deviance of predicted/actual edagger (see [edagger()]) as proposed in Rabbi & Mazzuco, 2021.
#' }
#'
#' @references
#'
#' Booth, H., Maindonald, J., & Smith, L. (2002). Applying Lee-Carter under conditions of variable mortality decline. Population Studies, 56(3), 325-336. \doi{10.1080/00324720215935}
#'
#' Lee, R. D., & Carter, L. R. (1992). Modeling and forecasting US mortality. Journal of the American Statistical Association, 87(419), 659–671. \doi{10.1080/01621459.1992.10475265}
#'
#' Lee, R., & Miller, T. (2001). Evaluating the performance of the Lee-Carter method for forecasting mortality. Demography, 38(4), 537–549. \doi{10.1353/dem.2001.0036}
#'
#' Li, N., Lee, R., & Gerland, P. (2013). Extending the Lee-Carter Method to Model the Rotation of Age Patterns of Mortality Decline for Long-Term Projections. Demography, 50(6), 2037–2051. \doi{10.1007/s13524-013-0232-2}
#'
#' Rabbi, A. M. F., & Mazzuco, S. (2021). Mortality forecasting with the Lee-Carter method: Adjusting for smoothing and lifespan disparity. European Journal of Population, 37(1), 97-120. \doi{10.1007/s10680-020-09559-9}
#'
#' Ševčíková, H., Li, N., Kantorová, V., Gerland, P., & Raftery, A. E. (2016). Age-Specific Mortality and Fertility Rates for Probabilistic Population Projections. In R. Schoen (Ed.), Dynamic Demographic Analysis (Vol. 39, pp. 285–310). Springer International Publishing. \doi{10.1007/978-3-319-26603-9_15}
#'
#' @return A list with four components: `model`, the fitted time-series model
#'   used to forecast `kt`; `kt`, a data frame with historical and forecast
#'   mortality index values and confidence intervals; `ex0`, a data frame with
#'   observed, fitted, and forecast life expectancy at birth and confidence
#'   intervals; and `mx`, a data frame with observed, fitted, and forecast
#'   age-specific mortality rates and confidence intervals.
#' @examples
#' age <- 0:20
#' year <- 2000:2009
#' lc_data <- expand.grid(age = age, year = year)
#' lc_data$mx <- exp(-7 + 0.09 * lc_data$age - 0.02 * (lc_data$year - min(year)))
#' fit <- leecart(lc_data, n = 2)
#' head(fit$kt)
#' @import forecast dplyr tidyr splines
#' @export
leecart <- function(data, n = 10, alpha = 0.05, model = "RWwD", ax_method = "classic", bx_method = "classic", boot = FALSE, bn = 1000, ktadj = "none", ...){

  ##### Checks
  `%notin%` <- Negate(`%in%`)
  if("year" %notin% colnames(data) | "age" %notin% colnames(data) | "mx"  %notin% colnames(data)){
    stop("Please, prepare your data carefully")
  }else{
    data <- data %>% arrange(year, age)
  }
  if(sum(data$mx == 0)>0){
    warning("0 are in mx vector, linear interpolation is done")
    mxtoextr <- which(data$mx == 0)
    for(j in mxtoextr){
      data1 <- data[data$year == data[j,]$year,]
      lh <- data1[c(j-1, j+1),]$mx
      data[j,]$mx <- mean(lh)
      data[j,]$mx <- ifelse(is.na(data[j,]$mx), 1e-5, 1e-5)
    }
  }

  ##### Model
  Mx <- data %>% select(age, year, mx) %>% pivot_wider(names_from = year, values_from = mx) %>% select(-age) %>% as.matrix() %>% log()

  if(ax_method == "classic"){
    ax <- rowMeans(Mx)
    Mx <- Mx - ax
    svd_result <- svd(Mx)
    U <- svd_result$u # left singular vectors - age
    D <- svd_result$d # singular values
    V <- svd_result$v # right singular vectors - years

    bx <- U[,1]
    kt <- D[1] * V[,1]

    # Normalization
    bx.sum = sum(bx)
    kt <- kt * bx.sum
    kt <- kt - mean(kt) # kt sum = 0
    bx <- bx / bx.sum # bx sum = 1
  }else if (ax_method == "last"){
    ax = Mx[,ncol(Mx)]
    Mx <- Mx - ax
    svd_result <- svd(Mx)
    U <- svd_result$u # left singular vectors - age
    D <- svd_result$d # singular values
    V <- svd_result$v # right singular vectors - years

    bx <- U[,1]
    kt <- D[1] * V[,1]

    # Normalization
    bx.sum = sum(bx)
    kt <- kt * bx.sum
    kt <- kt - kt[length(kt)] # kt = 0 in the last year
    bx <- bx / bx.sum # bx sum = 1
  }else if (ax_method == "last_smooth"){
    ax = Mx[,ncol(Mx)]
    ax[-1] <- stats::predict(stats::lm(Mx[-1,ncol(Mx)] ~ splines::ns(unique(data$age)[-1], df = round(length(unique(data$age))/5))))
    Mx <- Mx - ax
    svd_result <- svd(Mx)
    U <- svd_result$u # left singular vectors - age
    D <- svd_result$d # singular values
    V <- svd_result$v # right singular vectors - years

    bx <- U[,1]
    kt <- D[1] * V[,1]

    # Normalization
    bx.sum = sum(bx)
    kt <- kt * bx.sum
    kt <- kt - kt[length(kt)] # kt = 0 in the last year
    bx <- bx / bx.sum # bx sum = 1
  } else {
    stop("Choose `ax_method` arguement carefully")
  }

  if(ktadj == "Dmin"){

    if("Dx" %notin% colnames(data) | "N" %notin% colnames(data)){
      stop("for `ktadj = 'Dx'` adjustment in the data should be deaths (`Dx` column) and population (`N` column) by age")
    }

    Dx <- data %>% select(age, year, Dx) %>% pivot_wider(names_from = year, values_from = Dx) %>% select(-age) %>% as.matrix()
    Nx <- data %>% select(age, year, N) %>% pivot_wider(names_from = year, values_from = N) %>% select(-age) %>% as.matrix()

    fdmin <- function(k, ax, bx, Dx, Nx){
      sum(Dx) - sum(exp(ax + bx * k) * Nx)
    }
    kDx <- kt
    for(t in 1:ncol(Mx)){
      kDx[t] <- stats::uniroot(f = fdmin, interval = c(kt[t]-50, kt[t]+50), ax = ax, bx = bx, Dx = Dx[,t], Nx = Nx[,t], tol = 1e-6)$root[1]
    }
    kt <- kDx
  }else if (ktadj == "e0min"){

    e0 <- apply(exp(Mx + ax), 2, FUN = function(x) LT(age = unique(data$age), mx = x, ...)[1,"ex"])

    fe0 <- function(k, ax, bx, e0, ...){
      mx.hat <- exp(ax + bx * k)
      e0 - LT(age = unique(data$age), mx = mx.hat, ...)[1,"ex"]
    }
    kex <- kt
    for(t in 1:ncol(Mx)){
      kex[t] <- stats::uniroot(f = fe0, interval = c(kt[t]-50, kt[t]+50), ax = ax, bx = bx, e0 = e0[t], tol = 1e-6)$root[1]
    }
    kt <- kex
  } else if (ktadj == "poisson"){

    if("Dx" %notin% colnames(data) | "N" %notin% colnames(data)){
      stop("for `ktadj = 'poisson'` adjustment in the data should be deaths (`Dx` column) and population (`N` column) by age")
    }

    Dx <- data %>% select(age, year, Dx) %>% pivot_wider(names_from = year, values_from = Dx) %>% select(-age) %>% as.matrix()
    Nx <- data %>% select(age, year, N) %>% pivot_wider(names_from = year, values_from = N) %>% select(-age) %>% as.matrix()
    kpois <- kt
    for(t in 1:ncol(Mx)){
      kpois[t] <- stats::coef(stats::glm(round(Dx[,t]) ~ -1 + stats::offset(log(Nx[,t]) + ax) + bx, family = stats::poisson(link = "log")))[1]
    }
    kt <- kpois
  } else if (ktadj == "edaggermin"){

    edag <- apply(exp(Mx + ax), 2, FUN = function(x) edagger(age = unique(data$age), mx = x)[1])

    fedag <- function(k, ax, bx, edag, ...){
      mx.hat <- exp(ax + bx * k)
      (edag - edagger(age = unique(data$age), mx = mx.hat, ...)[1])^2
    }
    kedag <- kt
    for(t in 1:ncol(Mx)){
      kedag[t] <- stats::optimise(f = fedag, interval = c(kt[t]-50, kt[t]+50), ax = ax, bx = bx, edag = edag[t], tol = 1e-6)$minimum
    }
    kt <- kedag
  } else if (ktadj != "none"){
    stop("Choose `ktadj` arguement carefully")
  }

  ##### Fit
  mx.hat <- exp(ax + bx %o% kt)
  e0.hat <- apply(mx.hat, 2, FUN = function(x) LT(age = unique(data$age), mx = x, ...)[1,"ex"])

  ##### Forecast
  forecaster <- function(y = kt, mod = model){
    if(mod == "RWwD"){
      model <- forecast::Arima(y = y, order = c(0,1,0), include.drift = TRUE)
    }else if(mod == "ARIMA"){
      model <- forecast::auto.arima(y = y, seasonal = F)
    } else{
      stop("Choose `model` arguement carefully")
    }
    model
  }

  formodel <- forecaster()
  kt.for = forecast::forecast(formodel, bootstrap = boot, npaths = bn, h = n, level = 1-alpha) %>%
    as.data.frame()
  mx.for <- lapply(kt.for[,1], FUN = function(x) exp(ax + bx %o% x))
  mx.low <- lapply(kt.for[,3], FUN = function(x) exp(ax + bx %o% x))
  mx.high <- lapply(kt.for[,2], FUN = function(x) exp(ax + bx %o% x))

  e0.for <- sapply(mx.for, FUN = function(x) LT(age = unique(data$age), mx = x, ...)[1,"ex"])
  e0.low <- sapply(mx.low, FUN = function(x) LT(age = unique(data$age), mx = x, ...)[1,"ex"])
  e0.high <- sapply(mx.high, FUN = function(x) LT(age = unique(data$age), mx = x, ...)[1,"ex"])


  if(bx_method == "rotate"){
    bx2 = bx
    bx.m = mean(bx2[unique(data$age) %in% 15:64])
    bxu = ifelse(unique(data$age) <=64, bx.m, NA)
    bxu[is.na(bxu)] <- bx2[which(is.na(bxu))] * bx.m /  bx2[which(is.na(bxu))][1]
    bxu = bxu / sum(bxu)

    wt0.hat = (e0.hat - 80)/(102 - e0.hat)
    wt.hat = sqrt( 0.5 * ( 1 + sin( 0.5*pi*(2*wt0.hat - 1) ) ) )

    wt0.for = (e0.for - 80)/(102 - e0.for)
    wt.for = sqrt( 0.5 * ( 1 + sin( 0.5*pi*(2*wt0.for - 1) ) ) )

    wt0.low = (e0.low - 80)/(102 - e0.low)
    wt.low = sqrt( 0.5 * ( 1 + sin( 0.5*pi*(2*wt0.low - 1) ) ) )

    wt0.high = (e0.high - 80)/(102 - e0.high)
    wt.high = sqrt( 0.5 * ( 1 + sin( 0.5*pi*(2*wt0.high - 1) ) ) )

    for(t in 1:length(kt)){
      if(e0.hat[t] < 80){
        bxi = bx
      }else if(e0.hat[t] >= 80 & e0.hat[t] < 102){
        bxi = (1-wt.hat[t]) * bx + wt.hat[t] * bxu
      }else if(e0.hat[t] >= 102){
        bxi = bxu
      }
      mx.hat[,t] <- exp(ax + bxi %o% kt[t])
    }

    for(t in 1:n){
      if(e0.for[t] < 80){
        bxi = bx
      }else if(e0.for[t] >= 80 & e0.for[t] < 102){
        bxi = (1-wt.for[t]) * bx + wt.for[t] * bxu
      }else if(e0.for[t] >= 102){
        bxi = bxu
      }
      mx.for[[t]] <- exp(ax + bxi %o% kt.for[t,1])

      if(e0.low[t] < 80){
        bxi = bx
      }else if(e0.low[t] >= 80 & e0.low[t] < 102){
        bxi = (1-wt.low[t]) * bx + wt.low[t] * bxu
      }else if(e0.low[t] >= 102){
        bxi = bxu
      }
      mx.low[[t]] <- exp(ax + bxi %o% kt.for[t,3])

      if(e0.high[t] < 80){
        bxi = bx
      }else if(e0.high[t] >= 80 & e0.high[t] < 102){
        bxi = (1-wt.high[t]) * bx + wt.high[t] * bxu
      }else if(e0.high[t] >= 102){
        bxi = bxu
      }
      mx.high[[t]] <- exp(ax + bxi %o% kt.for[t,2])
    }

    e0.hat <- apply(mx.hat, 2, FUN = function(x) LT(age = unique(data$age), mx = x, ...)[1,"ex"])
    e0.for <- sapply(mx.for, FUN = function(x) LT(age = unique(data$age), mx = x, ...)[1,"ex"])
    e0.low <- sapply(mx.low, FUN = function(x) LT(age = unique(data$age), mx = x, ...)[1,"ex"])
    e0.high <- sapply(mx.high, FUN = function(x) LT(age = unique(data$age), mx = x, ...)[1,"ex"])
  }

  ##### Merging
  kt.findat <- data.frame(year = unique(data$year), kt = kt, conf.low = NA, conf.high = NA) %>%
    rbind(x = ., y = data.frame(year = (max(data$year) + 1):(max(data$year) + n),
                                kt = kt.for[,1],
                                conf.low = kt.for[,2],
                                conf.high = kt.for[,3])
    )

  e0.obs <- apply(exp(Mx + ax), 2, FUN = function(x) LT(age = unique(data$age), mx = x, ...)[1,"ex"])
  e0.findat <- data.frame(year = unique(data$year), e0.obs = e0.obs, e0.hat = e0.hat, conf.low = NA, conf.high = NA) %>%
    rbind(x = ., y = data.frame(year = (max(data$year) + 1):(max(data$year) + n),
                                e0.obs = NA,
                                e0.hat = e0.for,
                                conf.low = e0.low,
                                conf.high = e0.high)
    )


  mx.findat <- data.frame(year = lapply((max(data$year) + 1):(max(data$year) + n), FUN = function(x) rep(x, length(unique(data$age)))) %>% unlist(),
                          age = rep(unique(data$age), n),
                          mx.obs = NA,
                          mx.hat = unlist(mx.for),
                          conf.low = unlist(mx.high),
                          conf.high = unlist(mx.low)
  )

  mx.hat <- mx.hat %>% as.data.frame()
  colnames(mx.hat) = unique(data$year)
  mx.hat <- mx.hat %>%
    mutate(age = unique(data$age)) %>%
    pivot_longer(cols = !age,
                 names_to = "year",
                 values_to = "mx.hat"
    ) %>%
    arrange(year, age)
  mx.findat <- left_join(mx.hat %>% mutate(year = as.numeric(year)),
                         data %>% select(year, age, mx.obs = mx), by = c("year", "age")
  ) %>%
    mutate(conf.low = NA,
           conf.high = NA) %>%
    select(everything(colnames(mx.findat))) %>%
    rbind(x = ., y = mx.findat) %>%
    as.data.frame()


  return(
    list(
      model = formodel,
      kt = kt.findat %>% mutate(conf.low = ifelse(year == max(data$year), kt, conf.low), conf.high = ifelse(year == max(data$year), kt, conf.high)),
      ex0 = e0.findat %>% mutate(conf.low = ifelse(year == max(data$year), e0.hat, conf.low), conf.high = ifelse(year == max(data$year), e0.hat, conf.high)),
      mx = mx.findat %>% mutate(conf.low = ifelse(year == max(data$year), mx.hat, conf.low), conf.high = ifelse(year == max(data$year), mx.hat, conf.high))
    )
  )
}

#' Mortality models for mx approximation
#'
#' @param mx Numeric vector of age specific mortality rates.
#' @param age Numeric vector of ages.
#' @param model Character. Model name to be estimated. Now "Gompertz" and "Brass" are supported.
#' @param standard.mx Numeric vector of age specific mortality rates for standard population. Default is `NULL`.
#' @param ... Used only for `Brass` model. Other parameters for the function [LT()] including `ax` (by default, the middle of the interval), `sex` (by default = `"m"` - males), `l0` (by default = 1).
#'
#' @return A list with two components: `model`, the fitted `lm` or `nls`
#'   object; and `predicted`, a data frame with columns `age` and `mx.pred`
#'   containing the fitted mortality schedule.
#' @import dplyr
#'
#' @details
#' This function runs least squares optimization of the selected mortality function using Gauss-Newton algorithm algorithm with 2000 maximum iterations
#' and 1e-07 as tolerance parameter. For "Gompertz" usual OLS estimator is used.
#' ## Gompertz model
#' The model is as follows: \deqn{m(age) = \alpha e^{\beta age}}
#' ## Brass model
#' The model is as follows: \deqn{y(age) = \alpha + \beta y^{S}(age)}
#' where \deqn{y(age) = \frac{1}{2} ln[\frac{q(age)}{1-q(age)}]}
#' and subscript S defines that function is for standard population. To get mx from qx usual formula is used:
#' \deqn{m(age) = \frac{q(age)}{n-q(age)(n-a(age))}} where n is the size of age interval and a(x) is a parameter from life table.
#'
#' @references Preston, S. H., Heuveline, P., & Guillot, M. (2001). Demography: Measuring and Modeling Population Processes. Blackwell Publishers.
#' @export
#'
#' @examples
#' age <- seq(40, 80, 10)
#' mx <- c(0.003, 0.005, 0.009, 0.018, 0.036)
#' standard.mx <- c(0.0025, 0.004, 0.007, 0.014, 0.03)
#' mort.approx(mx = mx, age = age, model = "Gompertz")
#' mort.approx(mx = mx, age = age, model = "Brass", standard.mx = standard.mx)
mort.approx <- function(mx, age, model = c("Brass", "Gompertz"), standard.mx = NULL, ...){

  if(length(age) != length(mx)){
    stop("age and mx do not have the same length")
  }
  if(!(model %in% c("Gompertz", "Brass"))){
    stop("model can be only 'Gompertz' or 'Brass'")
  }

  if(model == "Gompertz"){
    est <- stats::lm(log(mx) ~ age)
    predicted = stats::predict(est)
    predicted = data.frame(age = age, mx.pred = exp(predicted))

  }

  if(model == "Brass"){
    if(is.null(standard.mx)){
      stop("For Brass model standard mx are required")
    }
    if(length(standard.mx) != length(mx)){
      stop("Standard mx and mx do not have the same length")
    }
    qx <- demor::LT(age = age, mx = mx, ...)[,"qx"]
    qx <- qx[-length(qx)]
    qx.stand = demor::LT(age = age, mx = standard.mx, ...)[,"qx"]
    ax.stand =
      qx.stand <- qx.stand[-length(qx.stand)]
    stand.logit = 0.5*log(qx.stand/(1-qx.stand))

    est <- stats::nls( 0.5*log((qx/(1-qx))) ~ a + b*stand.logit, start = list(a = 0, b = 1))
    qx.pred = c(exp(2*stats::predict(est))/(1+exp(2*stats::predict(est))),1)
    ax.stand = demor::LT(age = age, mx = standard.mx, ...)[,"ax"]
    n = diff(age)
    n = c(n, mean(n))
    mx.pred = qx.pred/(n - qx.pred*(n - ax.stand))
    predicted = data.frame(age = age, mx.pred = mx.pred)

  }


  return(
    list(
      model = est,
      predicted = predicted
    )
  )

}

test_that("l0 scales lx/dx/Lx/Tx but leaves qx and ex invariant", {
  age <- 0:5; mx <- rep(0.01, 6)
  lt1 <- LT(age = age, mx = mx, l0 = 1)
  ltB <- LT(age = age, mx = mx, l0 = 1e5)

  expect_equal(lt1[, "qx"], ltB[, "qx"])
  expect_equal(lt1[, "ex"], ltB[, "ex"], tolerance = 1e-2)
  expect_false(all(abs(ltB[, "lx"] / lt1[, "lx"]-1) < 1e-3))
  expect_false(all(abs(ltB[, "dx"] / lt1[, "dx"]-1) < 1e-3))
  expect_false(all(abs(ltB[, "Lx"] / lt1[, "Lx"]-1) < 1e-3))
  expect_false(all(abs(ltB[, "Tx"] / lt1[, "Tx"]-1) < 1e-3))
})

test_that("LT input validation and mx<0 handling work", {
  expect_error(LT(age = 0:5, mx = 0:3), regexp = "length")
  age <- 0:5; mx <- c(-0.01, rep(0.01, 5))
  expect_warning(lt <- LT(age = age, mx = mx), regexp = "replace")
  expect_equal(as.numeric(lt[1, "mx"]), 0)  # negatives replaced with 0
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{rosbris.codes}
\alias{rosbris.codes}
\title{Rosbris' Region codes}
\format{
\subsection{A dataframe}{

\describe{
\item{n}{number of Region (sequence number)}
\item{name}{Region name}
\item{code}{Region unique code}
}
}
}
\source{
Russian Fertility and Mortality Database. Center for Demographic Research, Moscow (Russia). Available at https://www.nes.ru/demogr-fermort-data
}
\usage{
rosbris.codes
}
\description{
Rosbris' Region codes
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-mdecomp.R
\name{plot.mdecomp}
\alias{plot.mdecomp}
\title{Plot for mdecomp function}
\usage{
\method{plot}{mdecomp}(x, return.data = FALSE, ...)
}
\arguments{
\item{x}{A result of age and cause decomposition from \code{\link[=mdecomp]{mdecomp()}}.}

\item{return.data}{Should the data be returned (\code{TRUE}) or the ggplot2 plot (default, \code{FALSE}).}

\item{...}{Ignored.}
}
\value{
If \code{return.data = FALSE}, a \code{ggplot2} object with stacked bars of
age- and cause-specific contributions. If \code{return.data = TRUE}, a data
frame with columns \code{age}, \code{ex12}, and \code{group} used to build the plot.
}
\description{
Plot for mdecomp function
}
\examples{
data(mdecompex)
usa <- subset(mdecompex, cnt == "usa")
eng <- subset(mdecompex, cnt == "eng")
dec <- mdecomp(
  mx1 = list(
    all = usa$all,
    neoplasms = usa$neoplasms,
    circulatory = usa$circulatory
  ),
  mx2 = list(
    all = eng$all,
    neoplasms = eng$neoplasms,
    circulatory = eng$circulatory
  ),
  age = usa$age
)
plot(dec)
}
\seealso{
\code{\link[=mdecomp]{mdecomp()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edagger.R
\name{edagger}
\alias{edagger}
\title{e-dagger}
\usage{
edagger(age, mx, ...)
}
\arguments{
\item{age}{Numeric array of age intervals}

\item{mx}{Numeric array with age specific mortality rates.}

\item{...}{Optional. Additional arguments for \code{\link[=LT]{LT()}} function.}
}
\value{
A named numeric vector of the same length as \code{age}, where each
element is \verb{e^\\dagger_x}, the average remaining life years lost because of
death from age \code{x} onward.
}
\description{
e-dagger
}
\examples{
age <- 0:5
mx <- c(0.02, 0.01, 0.012, 0.015, 0.02, 0.03)
edagger(age, mx)[1:3]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MLT.R
\name{MLT}
\alias{MLT}
\title{Multiple Decrement Life Table}
\usage{
MLT(age, mx, ...)
}
\arguments{
\item{age}{Numeric array of age intervals; for full life table = \code{0:100}; for concise life table = \code{c(0:1, seq(5,85,5))}}

\item{mx}{List of numeric arrays. 1st array should be all-cause mx in the population, other arrays are cause-specific mx in the population.}

\item{...}{Other parameters for the function \code{\link[=LT]{LT()}} including \code{ax} (by default, the middle of the interval), \code{sex} (by default = \code{"m"} - males), \code{l0} (by default = 1).}
}
\value{
A numeric matrix extending the output of \code{\link[=LT]{LT()}}. In addition to the
standard life-table columns, for each cause \code{i} in \code{mx[-1]} it adds
\code{qx_i}, \code{dx_i}, \code{lx_i}, and \code{ex_no_i}, corresponding to cause-specific
death probabilities, deaths, survivors, and cause-deleted life expectancy.
}
\description{
Multiple Decrement Life Table
}
\examples{
data(asdtex)
mx_causes <- list(
  all = asdtex$all,
  neoplasms = asdtex$neoplasms,
  circulatory = asdtex$circulatory
)
MLT(age = asdtex$age, mx = mx_causes)[1:3, c("age", "mx", "qx_neoplasms", "ex_no_neoplasms")]
}
\seealso{
\code{\link[=LT]{LT()}} for usual life table calculation
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary-leslie.R
\name{summary.leslie}
\alias{summary.leslie}
\title{Leslie matrix summary}
\usage{
\method{summary}{leslie}(object, d = 1, ...)
}
\arguments{
\item{object}{A leslie matrix from \code{\link[=leslie]{leslie()}}.}

\item{d}{Time step. By default it is 1. Only affects \eqn{r}.}

\item{...}{Ignored.}
}
\value{
A list of class \code{summary.leslie} with four components:
\code{lambda} (dominant eigenvalue / asymptotic growth factor), \code{r}
(intrinsic growth rate), \code{w} (stable age distribution summing to 1),
and \code{v} (reproductive values normalized so that \code{sum(v * w) = 1}).
}
\description{
Leslie matrix summary
}
\details{
The function calculates \eqn{\lambda}, \eqn{r}, \eqn{w} and \eqn{v}.
\itemize{
\item \eqn{\lambda} -- asymptotic growth factor that is dominant eigenvalue.
\item \eqn{r} -- asymptotic growth rate that is \eqn{ln \lambda / \delta t}.
\item \eqn{w} -- stable age distribution normalized to 1 s.t. \eqn{\sum_x^{\omega} w_x = 1} where \eqn{x} is age.
\item \eqn{v} -- reproductive values normalized s.t. \eqn{v'w = 1}.
}
}
\examples{
mx <- c(0.02, 0.01, 0.012, 0.015, 0.02)
fx <- c(0.05, 0.08)
les <- leslie(mx = mx, fx = fx, age.mx = 0:4, age.fx = 1:2)
summary(les)

}
\seealso{
\code{\link[=leslie]{leslie()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mac.R
\name{mac}
\alias{mac}
\title{Mean Age at Childbearing (MAC)}
\usage{
mac(fx, age)
}
\arguments{
\item{fx}{Numeric array of age specific fertility rates.}

\item{age}{Numeric array of ages. For example, \code{15:55} for 1-year age-groups}
}
\value{
A length-1 numeric value giving the mean age at childbearing implied
by \code{fx}.
}
\description{
Mean Age at Childbearing (MAC)
}
\examples{
age <- seq(15, 45, 5)
fx <- c(0.03, 0.10, 0.14, 0.12, 0.07, 0.03, 0.01)
mac(fx, age)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tatfr.R
\name{tatfr}
\alias{tatfr}
\title{Tempo-adjusted total fertility rate (TFR')}
\usage{
tatfr(past_fx, present_fx, post_fx, age)
}
\arguments{
\item{past_fx}{List with numeric arrays of age specific fertility rates for period t-1 by parity}

\item{present_fx}{List with numeric arrays of age specific fertility rates for period t by parity (it is period of interest)}

\item{post_fx}{List with numeric arrays of age specific fertility rates for period t+1 by parity}

\item{age}{Array with numeric values age}
}
\value{
A list with four components: \code{tatfr} (overall tempo-adjusted total
fertility rate), \code{tatfr_i} (parity-specific tempo-adjusted rates), \code{tfr}
(overall conventional TFR), and \code{tfr_i} (parity-specific conventional
rates).
}
\description{
Tempo-adjusted total fertility rate (TFR')
}
\details{
This indicator is calculated as follows
\deqn{TFR_{i,t}' = \frac{TFR_{i,t}}{1-(M_{i,t+1} - M_{i,t-1}) / 2}}
where \eqn{TFR_{i,t}', TFR_{i,t}} are tempo-adjusted and usual total fertility rate for parity \eqn{i} and time \eqn{t} respectively, \eqn{M_{i,t}} is mean age at childbearing for parity \eqn{i} and time \eqn{t}. The tempo-adjusted total fertility rate is a sum of parity-specific \eqn{TFR_i'}.

Note, the calculation are done as in footnote 1 in (Bongaarts & Feeney, 2000, p. 563). Unfortunately, the original 1998 article does not provide the exact formula, which has caused some confusion in academic circles.
}
\examples{
age <- seq(15, 45, 5)
past_fx <- list(
  c(0.02, 0.05, 0.07, 0.05, 0.03, 0.01, 0.00),
  c(0.01, 0.03, 0.04, 0.03, 0.02, 0.01, 0.00)
)
present_fx <- list(
  c(0.03, 0.06, 0.08, 0.06, 0.03, 0.01, 0.00),
  c(0.01, 0.03, 0.05, 0.04, 0.02, 0.01, 0.00)
)
post_fx <- list(
  c(0.03, 0.05, 0.08, 0.07, 0.04, 0.02, 0.00),
  c(0.01, 0.03, 0.04, 0.04, 0.03, 0.01, 0.00)
)
tatfr(past_fx, present_fx, post_fx, age)
}
\references{
Bongaarts, J., & Feeney, G. (1998). On the Quantum and Tempo of Fertility. Population and Development Review, 24(2), 271–291. \doi{10.2307/2807974}

Bongaarts, J., & Feeney, G. (2000). On the Quantum and Tempo of Fertility: Reply. Population and Development Review, 26(3), 560–564. \doi{10.1111/j.1728-4457.2000.00560.x}
}
\seealso{
\code{\link[=tfr]{tfr()}} for TFR and \code{\link[=mac]{mac()}} for mean age at childbearing calculation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tfr.R
\name{tfr}
\alias{tfr}
\title{Total Fertility Rate (TFR)}
\usage{
tfr(fx, age.int = 1)
}
\arguments{
\item{fx}{Numeric array of age specific fertility rates.}

\item{age.int}{Numeric. Age group: \code{1} for one-year, \code{5} for five-year. Any age groups are allowed.}
}
\value{
A length-1 numeric value equal to the sum of age-specific fertility
rates multiplied by \code{age.int}.
}
\description{
Total Fertility Rate (TFR)
}
\examples{
fx <- c(0.02, 0.08, 0.12, 0.09, 0.04, 0.01, 0.001)
tfr(fx, age.int = 5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/demor-package.R
\docType{package}
\name{demor-package}
\alias{demor}
\alias{demor-package}
\title{demor: Methods for Demographic Analysis}
\description{
Implements life tables, fertility and mortality indicators, decomposition methods, Lee-Carter mortality forecasting, Leslie matrices, and population pyramids for demographic analysis. Methods are described in Preston et al. (2001, ISBN:1557864519) and Ustyuzhanin (2025) \doi{10.17323/demreview.v12i4.30415}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://vadvu.github.io/demor/}
  \item \url{https://github.com/vadvu/demor}
  \item Report bugs at \url{https://github.com/vadvu/demor/issues}
}

}
\author{
\strong{Maintainer}: Vadim Ustyuzhanin \email{vadim.v.ustyuzhanin@gmail.com} (\href{https://orcid.org/0000-0003-3800-1108}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LT.R
\name{LT}
\alias{LT}
\title{Life table}
\usage{
LT(age, sex = "m", mx, ax = NULL, w = NULL, l0 = 1)
}
\arguments{
\item{age}{Numeric array of age intervals; for full life table = \code{0:100}; for concise life table = \code{c(0:1, seq(5,85,5))}}

\item{sex}{Character. Sex. \code{"m"} for males or \code{"f"} for females. By default = \code{"m"}.}

\item{mx}{Numeric array with age specific mortality rates.}

\item{ax}{Optional. Numeric array with ax. By default, it is the middle of the interval, while ax for age 0 is modeled as in Andreev & Kingkade (2015).}

\item{w}{Optional. Numeric array with weights for each age interval for calculating weighted life expectancy (\code{wex}).}

\item{l0}{Numeric. Life table radix. By default, = \code{1} but it can be any positive real number. In "human" demography tradition it is 100'000, in "ecological" and "evolutionary" demography tradition it is 1.}
}
\value{
A numeric matrix with one row per age group. Standard columns are
\code{age}, \code{mx}, \code{ax}, \code{qx}, \code{lx}, \code{dx}, \code{Lx}, \code{Tx}, and \code{ex}. If \code{w} is
supplied, additional columns \code{w}, \code{wLx}, and \code{wex} are appended, where
\code{wex} is weighted life expectancy.
}
\description{
Life table
}
\details{
By default, \eqn{a_x} for age 0 (first entity in \code{ax}) is modeled as in Andreev & Kingkade (2015, p. 390, see table 3-2).

The weighted life expectancy is calculated as follows:
\deqn{e_x^w = \frac{\sum_{i = x}^{\omega}L_x w_x}{l_x}}
where \eqn{\omega} is the last age, \eqn{w} is weight s.t. \eqn{w \in [0,1]}, and other variables are life table functions.
}
\examples{
# Minimal toy example
age <- 0:5
mx <- c(0.02, 0.01, 0.012, 0.015, 0.02, 0.03)
LT(age = age, sex = "m", mx = mx)
\donttest{
if (interactive()) {
  # Real RosBris data via get_rosbris(): Russian males, 2010
  rus2010 <- subset(get_rosbris("mortality_5"),
    year == 2010 & code == 1100 & sex == "m" & territory == "t"
  )
  LT(age = rus2010$age, sex = "m", mx = rus2010$mx)
}
}
}
\references{
Andreev, E. M., & Kingkade, W. W. (2015). Average age at death in infancy and infant mortality level: Reconsidering the Coale-Demeny formulas at current levels of low mortality. \emph{Demographic Research}, \emph{33}, 363-390.
}
\seealso{
\code{\link[=MLT]{MLT()}} for Multiple Decrement Life Table.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{asdtex}
\alias{asdtex}
\title{Data on mortality of US men in 2002 by some causes}
\format{
\subsection{A dataframe}{

\describe{
\item{age}{age}
\item{neoplasms}{age specific mortality rate (asmr) from Neoplasms}
\item{circulatory}{asmr from Diseases of the circulatory system}
\item{respiratory}{asmr from Diseases of the respiratory system}
\item{digestive}{asmr from Diseases of the digestive system}
\item{other}{asmr from other causes}
\item{all}{overall asmr}
}
}
}
\source{
Andreev & Shkolnikov spreadsheet, can be seen on this \href{https://www.demogr.mpg.de/en/publications_databases_6118/publications_1904/mpidr_technical_reports/an_excel_spreadsheet_for_the_decomposition_of_a_difference_between_two_values_of_an_aggregate_demographic_4591}{webpage}
}
\usage{
asdtex
}
\description{
Data on mortality of US men in 2002 by some causes
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gini.R
\name{gini}
\alias{gini}
\title{Gini coefficient of a life table}
\usage{
gini(age, mx, ...)
}
\arguments{
\item{age}{Numeric array of age intervals}

\item{mx}{Numeric array with age specific mortality rates.}

\item{...}{Optional. Additional arguments for \code{\link[=LT]{LT()}} function.}
}
\value{
A list with two components: \code{Gini}, itself a list with the relative
Gini coefficient \code{G0} and the absolute Gini coefficient \code{G0_abs}; and
\code{plot}, a data frame with columns \code{Fx} and \code{Phix} for drawing the Lorenz
curve.
}
\description{
Gini coefficient of a life table
}
\examples{
age <- 0:5
mx <- c(0.02, 0.01, 0.012, 0.015, 0.02, 0.03)
gini(age, mx)$Gini
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{sle_stand}
\alias{sle_stand}
\title{Data on Standard Life expectancies that is used for YLL calculations}
\format{
\subsection{A dataframe}{

\describe{
\item{stand}{Standard: 1 - World Health Organization Standard Life Expectancy by single-age; 2 - Global Burden of Disease studies (GBD) and WHO Global Health Estimates (WHO GHE) Standard Life Expectancy by 5-year age groups.}
\item{age}{age}
\item{ex}{Standard Life Expectancy}
}
}
}
\source{
Martinez, R., Soliz, P., Caixeta, R., Ordunez, P. (2019). Reflection on modern methods: years of life lost due to premature mortality-a versatile and comprehensive measure for monitoring non-communicable disease mortality. International Journal of Epidemiology, 48, 1367-1376. \doi{10.1093/ije/dyy254}
}
\usage{
sle_stand
}
\description{
Data on Standard Life expectancies that is used for YLL calculations
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/leslie.R
\name{leslie}
\alias{leslie}
\title{Leslie Matrix}
\usage{
leslie(mx, fx, age.mx, age.fx, srb = 100/205, fin = TRUE, ...)
}
\arguments{
\item{mx}{Numeric array of age specific mortality rates.}

\item{fx}{Numeric array of age specific fertility rates.}

\item{age.mx}{Numeric array of ages for mx.}

\item{age.fx}{Numeric array of ages for fx.}

\item{srb}{Numeric. Sex ratio at birth. Usually it is assumed that for males it is \code{105/205}, for females it is \code{100/205}. By default, it is \code{100/205}.}

\item{fin}{Logical. Should the survival rate for the last age-group be nonzero? By default it is \code{FALSE}, so the last survival rate is 0 as in classical model. Otherwise, it is \eqn{T_{x}/T_{x-1}}.}

\item{...}{Optional. Additional arguments for \code{\link[=LT]{LT()}} function.}
}
\value{
A square numeric matrix of class \code{leslie} with one row and one
column per age group. The first row contains fertility contributions and
the subdiagonal contains survival ratios.
}
\description{
Leslie Matrix
}
\examples{
mx <- c(0.02, 0.01, 0.012, 0.015, 0.02)
fx <- c(0.05, 0.08)
leslie(mx = mx, fx = fx, age.mx = 0:4, age.fx = 1:2)
}
\seealso{
\code{\link[=summary.leslie]{summary.leslie()}} for \code{leslie} output that calculates \eqn{\lambda}, \eqn{r}, \eqn{w} and \eqn{v}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ages.R
\name{ages}
\alias{ages}
\title{Split continuous ages to age groups}
\usage{
ages(x, groups, char = FALSE, below_min_val = NA)
}
\arguments{
\item{x}{Numeric array. Values of age that should be grouped}

\item{groups}{Numeric array. Values of lower boundaries of age groups}

\item{char}{Logical. Should output be numeric (\code{FALSE}) or character (\code{TRUE})? By default, \code{FALSE}}

\item{below_min_val}{What value to return for x < min(groups) when char = \code{FALSE}? Default is NA.}
}
\value{
If \code{char = TRUE}, a factor of the same length as \code{x} with age-group
labels. Otherwise, a numeric vector of the same length as \code{x} with the
lower boundaries from \code{groups}; values below the first group receive
\code{below_min_val}.
}
\description{
Split continuous ages to age groups
}
\examples{
groups <- c(0, 1, 5, 10, 15, 20, 24, 45, 85)
x <- 0:100
ages(x, groups, TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/med.age.R
\name{med.age}
\alias{med.age}
\title{Median age calculation}
\usage{
med.age(N, age)
}
\arguments{
\item{N}{Numeric array. Population counts by age groups (from young to old)}

\item{age}{Numeric array. Lower bounds of age groups, same length as \code{N}}
}
\value{
A length-1 numeric value giving the estimated median age of the
population represented by \code{N}.
}
\description{
Median age calculation
}
\examples{
N <- c(100, 90, 80, 70, 60)
age <- seq(0, 20, 5)
med.age(N, age)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mdecomp.R
\name{mdecomp}
\alias{mdecomp}
\title{Age and cause decomposition of differences in life expectancies}
\usage{
mdecomp(mx1, mx2, age, method = "andreev", ...)
}
\arguments{
\item{mx1}{List of numeric arrays. 1st array should be all-cause nmx in the 1st population, other arrays are cause-specific nmx in the 1st population}

\item{mx2}{List of numeric arrays. 1st array should be all-cause nmx in the 2nd population, other arrays are cause-specific nmx in the 2nd population}

\item{age}{Numeric array of age intervals; for full life table = \code{0:100}; for concise life table = \code{c(0:1, seq(5,85,5))}}

\item{method}{Character. Decomposition method. "andreev" (1982) or "arriaga" (1984) - slightly different in their results. By default, \code{method = "andreev"}.}

\item{...}{Optional. Additional arguments for \code{\link[=decomp]{decomp()}}.}
}
\value{
A data frame of class \code{c("mdecomp", "data.frame")} with one row per
age group. Column \code{age} contains ages, \code{ex12} contains the overall age
contribution to the life-expectancy difference, and the remaining columns
contain cause-specific contributions named after the cause-specific
elements of \code{mx1} and \code{mx2}.
}
\description{
Age and cause decomposition of differences in life expectancies
}
\details{
The contribution of each cause \eqn{c} to the absolute difference in life expectancies between the first and second population is caculated as
\deqn{\Delta_{x,c} = \frac{m^1_{x,c} - m^2_{x,c}}{m^1_{x} - m^2_{x}} \times \Delta_{x}}
where \eqn{\Delta_{x}} is contribution of age \eqn{x} to difference \eqn{e_0^2 - e_0^1} from function \code{\link[=decomp]{decomp()}}, \eqn{m^i_{x,c}} is age-specific mortality rate for population \eqn{i} from cause \eqn{c}, and \eqn{m^i_x} is total age-specific mortality rate.
}
\examples{
data(mdecompex)
usa <- subset(mdecompex, cnt == "usa")
eng <- subset(mdecompex, cnt == "eng")
dec <- mdecomp(
  mx1 = list(
    all = usa$all,
    neoplasms = usa$neoplasms,
    circulatory = usa$circulatory
  ),
  mx2 = list(
    all = eng$all,
    neoplasms = eng$neoplasms,
    circulatory = eng$circulatory
  ),
  age = usa$age
)
dec[1:3, ]
}
\seealso{
\code{\link[=decomp]{decomp()}} for just age decomposition and \code{\link[=plot.mdecomp]{plot.mdecomp()}} for graph of \code{mdecomp} results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_rosbris.R
\name{get_rosbris}
\alias{get_rosbris}
\title{Download RosBris Data}
\source{
Russian Fertility and Mortality Database. Center for Demographic
Research, Moscow (Russia). Available at
\url{https://www.nes.ru/research-main/research-centers/demogr/demogr-fermort-data}
}
\usage{
get_rosbris(
  dataset = c("mortality_1", "mortality_5", "fertility_1", "fertility_5"),
  refresh = FALSE
)
}
\arguments{
\item{dataset}{Character. Dataset to download and parse. One of
\code{"mortality_1"}, \code{"mortality_5"}, \code{"fertility_1"}, or \code{"fertility_5"}.}

\item{refresh}{Logical. If \code{FALSE} (default), previously downloaded RosBris
archives are reused from the local cache directory. If \code{TRUE}, archives are
downloaded again from the RosBris website.}
}
\value{
A data frame with one row per year-region-age combination. Column
structure depends on \code{dataset}; see \strong{Details}.
}
\description{
Download age-specific mortality, fertility, and population data from the
Russian Fertility and Mortality Database (RosBris) and return them as
long-format data frames for use in \code{demor}.
}
\details{
The function downloads official \code{.zip} archives from the RosBris website,
stores them in a user cache directory created by \code{tools::R_user_dir()}, reads
\code{.txt} tables from the archives, and converts them to long-format data frames.

Returned data have the following structure:
\itemize{
\item \code{"mortality_1"}: \code{year}, \code{code}, \code{territory}, \code{sex}, \code{age}, \code{mx}, \code{N},
\code{Dx}, \code{name}.
\item \code{"mortality_5"}: \code{year}, \code{code}, \code{territory}, \code{sex}, \code{age}, \code{mx}, \code{N},
\code{Dx}, \code{name}.
\item \code{"fertility_1"}: \code{year}, \code{code}, \code{territory}, \code{age}, \code{fx}, \code{N}, \code{Bx},
\code{name}.
\item \code{"fertility_5"}: \code{year}, \code{code}, \code{territory}, \code{age}, \code{fx}, \code{fx1}, \code{fx2},
\code{fx3}, \code{fx4}, \code{fx5}, \code{N}, \code{Bx}, \code{Bx1}, \code{Bx2}, \code{Bx3}, \code{Bx4}, \code{Bx5},
\code{name}.
}

At the moment, \code{get_rosbris()} uses the legacy RosBris series corresponding
to the periods \code{1989-2014} and \code{2015-2022}. Updated post-census series are
not used by this function.
}
\examples{
\donttest{
if (interactive()) {
  mort <- get_rosbris("mortality_5")
  rus2010 <- subset(
    mort,
    year == 2010 & code == 1100 & sex == "m" & territory == "t"
  )
  head(rus2010)
}
}

}
\seealso{
\link{rosbris.codes}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/yll.R
\name{yll}
\alias{yll}
\title{Years of Life Lost (YLL) calculation}
\usage{
yll(
  Dx,
  type = c("yll", "yll.p", "yll.r", "asyr"),
  age.int = 5,
  Dx_all = NULL,
  pop = NULL,
  w = NULL,
  standard = NULL
)
}
\arguments{
\item{Dx}{Array with the number of deaths.}

\item{type}{Character. Type of YLL to calculate. See details section.}

\item{age.int}{Numeric. Age interval of Dx. Can be \code{1} or \code{5}.}

\item{Dx_all}{Array with the number of all deaths. Used only with \code{yll.p} type, where \code{Dx} is array with cause-specific deaths.}

\item{pop}{Array with population. Used only with \code{yll.r} and \code{asyr} types.}

\item{w}{Array with population weights for direct standardization. Used only with \code{asyr} type.}

\item{standard}{Data frame. User-specific standard life expectancy to calculate YLL with the following columns: age, ex. Note: the \code{age.int} argument should be consistent with the \code{age} column in this data frame.}
}
\value{
A list whose components depend on \code{type}. For \code{type = "yll"}, the
list contains \code{yll_all} (overall YLL) and \code{yll} (age-specific YLL). For
\code{type = "yll.p"}, it contains \code{yll.p_all} and \code{yll.p} (overall and
age-specific YLL proportions). For \code{type = "yll.r"}, it contains
\code{yll.r_all} and \code{yll.r} (overall and age-specific YLL rates). For
\code{type = "asyr"}, it contains \code{asyr_all} and \code{asyr} (overall and
age-specific age-standardized YLL rates).
}
\description{
Years of Life Lost (YLL) calculation
}
\details{
Computes four types of Years of Life Lost (YLL) indicators:
\itemize{
\item \strong{Absolute YLL} (\code{type = "yll"}):
\deqn{YLL_{x,t,c} = D_{x,t,c} \times SLE_x}
where \eqn{x,t,c} are age, time, and cause respectively, \eqn{D_{x,t,c}} is deaths in age \eqn{x} at time \eqn{t} from cause \eqn{c} and \eqn{SLE_x} is standard life expectancy at age \eqn{x}
\item \strong{YLL proportion} (\code{type = "yll.p"}):
\deqn{YLL_{x,t,c}^p = \frac{YLL_{x,t,c}}{YLL_{x,t}}}
where \eqn{YLL_{x,t} = \sum_{c} YLL_{x,t,c}} (total YLL across causes)

\item \strong{YLL rate} (\code{type = "yll.r"}):
\deqn{YLL_{x,t,c}^r = \left( \frac{YLL_{x,t,c}}{N_{x,t}} \right) \times 100'000}
where \eqn{N_{x,t}} is population in age group \eqn{x} at time \eqn{t}

\item \strong{Age-standardized YLL rate} (\code{type = "asyr"}):
\deqn{ASYR_{t,c} = \sum_{x=\alpha}^{\omega} \left( YLL_{x,t,c}^r \times w_x \right)}
where \eqn{\alpha} to \eqn{\omega} corresponds to first to last age group, \eqn{w_x} is standard population weight for age \eqn{x}.
}
}
\examples{
yll(Dx = rep(1, 19), type = "yll", age.int = 5)$yll_all
}
\references{
Martinez, R., Soliz, P., Caixeta, R., & Ordunez, P. (2019). Reflection on modern methods: years of life lost due to premature mortality—a versatile and comprehensive measure for monitoring non-communicable disease mortality. \emph{International Journal of Epidemiology}, \emph{48}, 1367–1376.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_pyr.R
\name{plot_pyr}
\alias{plot_pyr}
\title{Plot population pyramid}
\usage{
plot_pyr(
  popm,
  popf,
  popm2 = NULL,
  popf2 = NULL,
  age,
  prc = FALSE,
  sexn = c("Males", "Females"),
  sexc = c("#ED0000B2", "#00468BB2"),
  age.cont = NULL,
  un.intervals = TRUE
)
}
\arguments{
\item{popm}{Numeric vector of male population}

\item{popf}{Numeric vector of female population}

\item{popm2}{Optional. Numeric vector of second male population to be plotted as line.}

\item{popf2}{Optional. Numeric vector of second female population to be plotted as line.}

\item{age}{Numeric vector of ages}

\item{prc}{Should the population be shown as a percentage rather than in absolute terms? By default, it is \code{FALSE} and the plot shows user-specific values in \code{popm} and \code{popf}.}

\item{sexn}{Character array of labels for the sexes. By default = \code{c("Males", "Females")}}

\item{sexc}{Character array of colors for the sexes. By default = \code{c("#ED0000B2","#00468BB2")}, which are red and blue}

\item{age.cont}{Optional. Logical. Should the age axis be considered as continuous scale? Recommend to switch to \code{TRUE} if the age intervals are small (1 year) and \code{FALSE} otherwise (for ex., when age interval is 5 years). By default function chooses the value by itself.}

\item{un.intervals}{Logical. Should the age groups be unified? For example, if the first and second groups are 0-1, 1-4, and all other groups are five-year, the function automatically makes the group 1-4 to make all the intervals the same. By default, \code{TRUE}.}
}
\value{
A \code{ggplot2} object representing a population pyramid. If \code{popm2} and
\code{popf2} are supplied, the plot also includes dashed comparison lines for
the second population.
}
\description{
Plot population pyramid
}
\examples{
plot_pyr(
  popm = c(100, 90, 80, 70, 60),
  popf = c(95, 92, 85, 75, 65),
  age = seq(0, 20, 5)
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccm.R
\name{ccm}
\alias{ccm}
\title{Cohort Component Model for Projecting Population.}
\usage{
ccm(
  Mx.f,
  Mx.m = NULL,
  Fx,
  Ix.f = NULL,
  Ix.m = NULL,
  age.mx,
  age.fx,
  N0.f,
  N0.m = NULL,
  srb = 100/205,
  ...
)
}
\arguments{
\item{Mx.f}{Matrix or dataframe with mx for females, where row represents age, column represents a forecasted period. Thus, first column should be the first projected period, while each row represents age-specific mortality rate.}

\item{Mx.m}{Optional. Matrix or dataframe with mx for males, where row represents age, column represents a forecasted period. Thus, first column should be the first projected period, while each row represents age-specific mortality rate.}

\item{Fx}{Matrix or dataframe with fx, where row represents age, column represents a forecasted period. Thus, first column should be the first projected period, while each row represents age-specific fertility rate.}

\item{Ix.f}{Optional. Matrix or dataframe with net number of female migrants, where row represents age, column represents a forecasted period. Thus, first column should be the first projected period, while each row represents net number of migrants.}

\item{Ix.m}{Optional. Matrix or dataframe with net number of male migrants, where row represents age, column represents a forecasted period. Thus, first column should be the first projected period, while each row represents net number of migrants.}

\item{age.mx}{Numeric vector. Age groups for Mx.f rows.}

\item{age.fx}{Numeric vector. Age groups for Fx rows.}

\item{N0.f}{Numeric vector. Female population in the initial period by age groups.}

\item{N0.m}{Optional. Numeric vector. Male population in the initial period by age groups.}

\item{srb}{Numeric. Sex ratio at birth for females. By default, it is 100/205.}

\item{...}{Optional. Additional arguments for \code{\link[=leslie]{leslie()}} function.}
}
\value{
If \code{Mx.m} is \code{NULL}, a numeric matrix with projected female
population, where rows are age groups and columns are periods \code{0:h}. If
male mortality is supplied, a list with matrices \code{female}, \code{male}, and
\code{all}, each with the same row/column structure.
}
\description{
Cohort Component Model for Projecting Population.
}
\details{
The model is calculated in matrix form as
\deqn{\mathbf{N}_{t+h} = \mathbf{L}_t(\mathbf{N}_{t} + \mathbf{I}_{t}/2) + \mathbf{I}_{t}/2}
where \eqn{\mathbf{N}_t} is a column vector of population for time \eqn{t} with \eqn{h} as a step of projection (it is the length of age interval), \eqn{\mathbf{L}_t} is Leslie matrix and \eqn{\mathbf{I}_{t}} is a column vector of net migration.

Note that the model assumes that in \eqn{\mathbf{N}} all age intervals are the same (i.e. \eqn{age = \{0-1, 1-4, 5-9, ...\}} is not permitted). Fortunately (and thanks to me), the function handles such situations automatically by transforming all age groups into a unified standard.
}
\examples{
age.mx <- seq(0, 80, 5)
age.fx <- seq(15, 45, 5)
Mx.f <- matrix(
  rep(seq(0.005, 0.12, length.out = length(age.mx)), 2),
  nrow = length(age.mx),
  ncol = 2
)
Fx <- matrix(
  rep(c(0.02, 0.08, 0.11, 0.09, 0.05, 0.02, 0.005), 2),
  nrow = length(age.fx),
  ncol = 2
)
N0.f <- round(100000 * exp(-0.04 * age.mx))
ccm(Mx.f = Mx.f, Fx = Fx, age.mx = age.mx, age.fx = age.fx, N0.f = N0.f)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fert.approx.R
\name{fert.approx}
\alias{fert.approx}
\title{Fertility models for ASFR approximation}
\usage{
fert.approx(fx, age, model, start = NULL, se = FALSE, alpha = 0.05, bn = 1000)
}
\arguments{
\item{fx}{Numeric vector of age specific fertility rates.}

\item{age}{Numeric vector of ages.}

\item{model}{Character. Model name to be estimated. Now "Hadwiger", "Gamma", "Brass" and "Beta" are supported.}

\item{start}{Numeric vector with user-specific values of parameters for optimization. Default is \code{NULL} (choose automatically)}

\item{se}{Logical. Should bootstrapped variance for ASFR approximation be calculated. Default is \code{FALSE} for no bootstrap.}

\item{alpha}{Numeric. Used if \code{se = TRUE}, the level of uncertainty. By default, \code{alpha = 0.05} for 95\% CI.}

\item{bn}{Numeric. Used if \code{se = TRUE}, number of bootstrap samples. By default, \code{bn = 1000}.}
}
\value{
A list with two components: \code{model}, a list describing the fitted
fertility model (\code{type}, fitted \code{params}, \code{rmse}, and, if \code{se = TRUE},
bootstrap \code{covmat} and parameter percentile intervals \code{prc}); and
\code{predicted}, a data frame with observed and fitted age-specific fertility
rates. When \code{se = TRUE}, \code{predicted} also includes bootstrap standard
errors and percentile intervals.
}
\description{
Fertility models for ASFR approximation
}
\details{
This function runs least squares optimization (using default \code{optim}) of the selected fertility function with 1e-06 as tolerance parameter.

\eqn{f_x} is age-specific fertility rate for age \eqn{x}.
\subsection{Hadwiger model}{

The model is as follows: \deqn{f_x = \frac{ab}{c} \frac{c}{x}^{3/2} exp[-b^2(\frac{c}{x}+\frac{x}{c}-2)]}
where \eqn{a,b,c} are estimated parameters that do not have demographic interpretation. Sometimes \eqn{c} is interpreted as mean age at childbearing.
}

\subsection{Gamma model}{

The model is as follows: \deqn{f_x = \frac{R}{\Gamma(b)c^b}(x-d)^{b-1} exp[-(\frac{x-d}{c})]}
where \eqn{R,b,c,d} are estimated parameters. \eqn{\Gamma} is gamma function. \eqn{R} can be interpreted as fertility level (TFR) and \eqn{d} as mean age at childbearing.
}

\subsection{Brass model}{

The model is as follows: \deqn{f_x = c(x-d)(d+w-x)}
where \eqn{c,d,w} are estimated parameters.
}

\subsection{Beta model}{

The model is as follows: \deqn{f_x = \frac{R}{B(A,C)}(\beta - \alpha)^{-(A+C-1)}(x-\alpha)^{(A-1)}(\beta-x)^{(B-1)}}
where \eqn{B(A, C)} is beta function, \eqn{R, \beta, \alpha} are estimated parameters, which can be interpreted as fertility level (TFR) and max and min age of childbearing respectively.
\eqn{A,C} are
\deqn{C = (\frac{(v - \alpha)(\beta - v)}{\tau^2} - 1)\frac{\beta - v}{\beta - \alpha}}
\deqn{A = C\frac{v-\alpha}{v - \beta}}
where \eqn{v, \tau^2} are estimated parameters, where \eqn{v} can be interpreted as mean age at childbearing. Thus, Beta model uses 5 parameters \eqn{R, \beta, \alpha, v, \tau^2}, where only \eqn{\tau^2} has no demographic interpretation.
}
}
\examples{
age <- seq(15, 45, 5)
fx <- c(0.03, 0.10, 0.14, 0.12, 0.07, 0.03, 0.01)
fert.approx(fx = fx, age = age, model = "Hadwiger", se = FALSE)
}
\references{
Peristera, P., & Kostaki, A. (2007). Modeling fertility in modern populations. \emph{Demographic Research}, \emph{16}, 141-194.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asdt.R
\name{asdt}
\alias{asdt}
\title{Associated single decrement life table (ASDT) for causes of death (cause-deleted life table)}
\usage{
asdt(age, m_all, m_i, full = FALSE, method = "chiang1968", ...)
}
\arguments{
\item{age}{Numeric array of age intervals; for full life table = \code{0:100}; for concise life table = \code{c(0:1, seq(5,85,5))}}

\item{m_all}{Numeric array with age specific mortality rates of all causes of death (usual mx).}

\item{m_i}{Numeric array with age specific mortality rates of some cause of death (i)}

\item{full}{Logical. Is full table needed? \code{TRUE} for full, \code{FALSE} for concise. By default, \code{FALSE}}

\item{method}{Character. The method of ASDT construction to use. Now just "chiang1968" is supported.}

\item{...}{Optional. Additional arguments for \code{\link[=LT]{LT()}} function.}
}
\value{
If \code{full = TRUE}, a data frame with the full associated single
decrement life table, including the standard life-table columns and the
cause-deleted columns \code{r_not_i}, \code{p_not_i}, \code{l_not_i}, \code{a_not_i},
\code{d_not_i}, \code{L_not_i}, \code{T_not_i}, and \code{ex_without_i}. If \code{full = FALSE}, a
reduced data frame with columns \code{age}, \code{r_not_i}, \code{lx}, \code{qx}, \code{ax}, \code{ex},
\code{p_not_i}, \code{l_not_i}, \code{a_not_i}, and \code{ex_without_i}.
}
\description{
Associated single decrement life table (ASDT) for causes of death (cause-deleted life table)
}
\examples{
data(asdtex)
asdt(
  age = asdtex$age,
  m_all = asdtex$all,
  m_i = asdtex$circulatory
)[1:3, ]
}
\references{
Chiang, L. (1968). \emph{Introduction to Stochastic Processes in Biostatistics}. New York: John Wiley and Sons.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decomp.R
\name{decomp}
\alias{decomp}
\title{Age decomposition of mortality: single decrement process}
\usage{
decomp(mx1, mx2, sex = "m", age, method = "andreev", ax1 = NULL, ax2 = NULL)
}
\arguments{
\item{mx1}{Numeric array with age specific mortality rates of population 1 (base population).}

\item{mx2}{Numeric array with age specific mortality rates of population 2 (compared population).}

\item{sex}{Character. Sex. "m" for males and "f" for females. By default, \code{sex = "m"}.}

\item{age}{Numeric array of age intervals; for full life table = \code{0:100}; for concise life table = \code{c(0:1, seq(5,85,5))}}

\item{method}{Character. Decomposition method. "andreev" (1982), "arriaga" (1984) or "pollard" (1982) - slightly different in their results. By default, \code{method = "andreev"}.}

\item{ax1}{Optional. Numeric array with ax for the 1st population. By default, it is a the middle of the interval, while ax for age 0 is modeled as in Andreev & Kingkade (2015).}

\item{ax2}{Optional. Numeric array with ax for the 2nd population. By default, it is a the middle of the interval, while ax for age 0 is modeled as in Andreev & Kingkade (2015).}
}
\value{
A data frame with one row per age group. It always contains \code{age},
\code{ex12}, and \code{ex12_prc}, where \code{ex12} is the absolute age contribution to
the life-expectancy difference and \code{ex12_prc} is the percentage
contribution. Additional columns depend on \code{method} and contain
intermediate life-table quantities used in the decomposition.
}
\description{
Age decomposition of mortality: single decrement process
}
\details{
Example of decomposition using Andreev (1982) formulas:
\deqn{\Delta_x = l_x^2(e_x^2 - e_x^1) - l_{x+n}^2(e_{x+n}^2 - e_{x+n}^1), \ \ \ x \neq \omega}
\deqn{\Delta_{\omega} = l_{\omega}^2(e_{\omega}^2 - e_{\omega}^1)}
where \eqn{\Delta_x} is an absolute contribution of age \eqn{x} to difference in \eqn{e_0} between the second and the first population. \eqn{e_x^i, l_x^i} are life table functions for population \eqn{i}. \eqn{\omega} is the last age group. Note, \eqn{e_0^2 - e_0^1 = \sum_{x}^{\omega}\Delta_x}
}
\examples{
age <- 0:5
mx1 <- c(0.02, 0.01, 0.012, 0.015, 0.02, 0.03)
mx2 <- c(0.018, 0.009, 0.011, 0.014, 0.019, 0.028)
decomp(mx1, mx2, age = age)$ex12
}
\references{
\enumerate{
\item Arriaga, E. E. (1984). Measuring and explaining the change in life expectancies. \emph{Demography}, \emph{21}, 83-96.
\item Andreev, E. M. (1982). Metod komponent v analize prodolzhitel'nosti zhizni. \emph{Vestnik statistiki}, \emph{9}, 42-47.
\item Pollard, J. H. (1982). The expectation of life and its relationship to mortality. \emph{Journal of the Institute of Actuaries}, \emph{109(2)}, 225–240.
}
}
\seealso{
\code{\link[=mdecomp]{mdecomp()}} for age and cause decomposition
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hli.R
\name{hli}
\alias{hli}
\title{The Human Life Indicator (HLI)}
\usage{
hli(age, mx, ...)
}
\arguments{
\item{age}{Numeric array of age intervals; for full life table = \code{0:100}; for concise life table = \code{c(0:1, seq(5,85,5))}}

\item{mx}{Numeric array with age specific mortality rates.}

\item{...}{Optional. Additional arguments for \code{\link[=LT]{LT()}} function.}
}
\value{
A length-1 numeric value giving the Human Life Indicator, i.e. the
geometric mean age at death implied by the life table.
}
\description{
The Human Life Indicator (HLI)
}
\details{
It is calculated as
\deqn{HLI = \prod_{x=\alpha}^{\omega}(x + a_x)^{d_x}}
where \eqn{\alpha, \omega} are the first and last age groups, \eqn{x} is age, \eqn{a_x, d_x} are life table functions (s.t. \eqn{\sum_{x=\alpha}^{\omega} d_x = 1}).
}
\examples{
age <- 0:5
mx <- c(0.02, 0.01, 0.012, 0.015, 0.02, 0.03)
hli(age, mx)
}
\references{
Ghislandi, S., Sanderson, W.C., & Scherbov, S. (2019). A Simple Measure of Human Development: The Human Life Indicator. \emph{Population and Development Review}, \emph{45}, 219–233.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/leecarter.R
\name{leecart}
\alias{leecart}
\title{Lee-Carter model}
\usage{
leecart(
  data,
  n = 10,
  alpha = 0.05,
  model = "RWwD",
  ax_method = "classic",
  bx_method = "classic",
  boot = FALSE,
  bn = 1000,
  ktadj = "none",
  ...
)
}
\arguments{
\item{data}{Dataframe in the long format with the following columns: \code{age}, \code{year}, \code{mx} (age specific mortality rates). For some types of \code{ktadj} argument \code{N} (population at age x) and \code{Dx} (number of deaths at age x) columns should also be presented.}

\item{n}{Numeric. Forecasted horizon}

\item{alpha}{Numeric. The level of uncertainty. By default, \code{alpha = 0.05} for 95\% CI.}

\item{model}{Character. Model type for kt forecasting. Can be "RWwD" for random walk with drift (by default, for original Lee-Carter model) or "ARIMA" for ARIMA model which parameters are chosen automatically by \code{forecast::auto.arima()}.}

\item{ax_method}{Character. Method for ax calculation. Can be "classic" from original Lee-Carter model (by default), "last" or "last_smooth". See details.}

\item{bx_method}{Character. Method for bx calculation. Can be "classic" from original Lee-Carter model (by default) and "rotate" for rotating bx (Li et al., 2013).}

\item{boot}{Logical. Should bootstrap estimates for uncertainty be used? \code{FALSE} by default.}

\item{bn}{Numeric. Used if \code{boot = TRUE}, number of bootstrap samples. By default, \code{bn = 1000}.}

\item{ktadj}{Character. Type of \code{kt} adjustment. It can be set to 'none' (defaukt, no adjustment), 'Dmin', 'e0min', 'poisson' or 'edaggermin' (see Details). Note that 'Dmin' and 'poisson' require data on the age-specific number of deaths (\code{Dx} column in the data) and the age-specific population (\code{N} column in the data).}

\item{...}{Optional. Additional arguments for \code{\link[=LT]{LT()}} function.}
}
\value{
A list with four components: \code{model}, the fitted time-series model
used to forecast \code{kt}; \code{kt}, a data frame with historical and forecast
mortality index values and confidence intervals; \code{ex0}, a data frame with
observed, fitted, and forecast life expectancy at birth and confidence
intervals; and \code{mx}, a data frame with observed, fitted, and forecast
age-specific mortality rates and confidence intervals.
}
\description{
Lee-Carter model
}
\details{
The \strong{\code{model}} argument specifies the forecasting method.
\itemize{
\item \code{model ="RWwD"} – classic random walk option
\item \code{model = "ARIMA"} for selecting a more complex time series model
}

The \strong{\code{ax_method}} argument allows to control how \code{a_x} is calculated.
\itemize{
\item \code{ax_method = "classic"} – classic option with the average of the logarithm of mortality rates (but there is so-called "jump-off bias").
\item \code{ax_method = "last"} uses the logarithm of mortality for the last available year (as proposed in Lee & Miller, 2001).
\item \code{ax_method = "last_smooth"} uses data for the last year with smoothing (see Ševčíková et al., 2016, p. 288).
}

The \strong{\code{bx_method}} argument allows to control how \code{b_x} is calculated.
\itemize{
\item \code{bx_method = "classic"} for the original method.
\item \code{bx_method = "rotate"} for the rotational variant (see Li et al., 2013).
}

The \strong{\code{ktadj}} argument allows to control how \code{k_t} is calculated.
\itemize{
\item \code{ktadj = "none"} for no adjustment.
\item \code{ktadj = "Dmin"} for minimizing the deviance of predicted/actual annual deaths (as proposed in the original Lee-Carter paper). This method requires data on the age-specific number of deaths (\code{Dx} column in the data) and the age-specific population (\code{N} column in the data).
\item \code{ktadj = "e0min"} for minimizing the deviance of predicted/actual life expectancy (as proposed in Lee & Miller, 2001).
\item \code{ktadj = "poisson"} for minimizing the deviance from a Poisson model, where the dependent variable is the age-specific annual number of deaths (as proposed in Booth et al., 2002). This method requires data on the age-specific number of deaths (\code{Dx} column in the data) and the age-specific population (\code{N} column in the data).
\item \code{ktadj = "edaggermin"} for minimizing the deviance of predicted/actual edagger (see \code{\link[=edagger]{edagger()}}) as proposed in Rabbi & Mazzuco, 2021.
}
}
\examples{
age <- 0:20
year <- 2000:2009
lc_data <- expand.grid(age = age, year = year)
lc_data$mx <- exp(-7 + 0.09 * lc_data$age - 0.02 * (lc_data$year - min(year)))
fit <- leecart(lc_data, n = 2)
head(fit$kt)
}
\references{
Booth, H., Maindonald, J., & Smith, L. (2002). Applying Lee-Carter under conditions of variable mortality decline. Population Studies, 56(3), 325-336. \doi{10.1080/00324720215935}

Lee, R. D., & Carter, L. R. (1992). Modeling and forecasting US mortality. Journal of the American Statistical Association, 87(419), 659–671. \doi{10.1080/01621459.1992.10475265}

Lee, R., & Miller, T. (2001). Evaluating the performance of the Lee-Carter method for forecasting mortality. Demography, 38(4), 537–549. \doi{10.1353/dem.2001.0036}

Li, N., Lee, R., & Gerland, P. (2013). Extending the Lee-Carter Method to Model the Rotation of Age Patterns of Mortality Decline for Long-Term Projections. Demography, 50(6), 2037–2051. \doi{10.1007/s13524-013-0232-2}

Rabbi, A. M. F., & Mazzuco, S. (2021). Mortality forecasting with the Lee-Carter method: Adjusting for smoothing and lifespan disparity. European Journal of Population, 37(1), 97-120. \doi{10.1007/s10680-020-09559-9}

Ševčíková, H., Li, N., Kantorová, V., Gerland, P., & Raftery, A. E. (2016). Age-Specific Mortality and Fertility Rates for Probabilistic Population Projections. In R. Schoen (Ed.), Dynamic Demographic Analysis (Vol. 39, pp. 285–310). Springer International Publishing. \doi{10.1007/978-3-319-26603-9_15}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mort.approx.R
\name{mort.approx}
\alias{mort.approx}
\title{Mortality models for mx approximation}
\usage{
mort.approx(mx, age, model = c("Brass", "Gompertz"), standard.mx = NULL, ...)
}
\arguments{
\item{mx}{Numeric vector of age specific mortality rates.}

\item{age}{Numeric vector of ages.}

\item{model}{Character. Model name to be estimated. Now "Gompertz" and "Brass" are supported.}

\item{standard.mx}{Numeric vector of age specific mortality rates for standard population. Default is \code{NULL}.}

\item{...}{Used only for \code{Brass} model. Other parameters for the function \code{\link[=LT]{LT()}} including \code{ax} (by default, the middle of the interval), \code{sex} (by default = \code{"m"} - males), \code{l0} (by default = 1).}
}
\value{
A list with two components: \code{model}, the fitted \code{lm} or \code{nls}
object; and \code{predicted}, a data frame with columns \code{age} and \code{mx.pred}
containing the fitted mortality schedule.
}
\description{
Mortality models for mx approximation
}
\details{
This function runs least squares optimization of the selected mortality function using Gauss-Newton algorithm algorithm with 2000 maximum iterations
and 1e-07 as tolerance parameter. For "Gompertz" usual OLS estimator is used.
\subsection{Gompertz model}{

The model is as follows: \deqn{m(age) = \alpha e^{\beta age}}
}

\subsection{Brass model}{

The model is as follows: \deqn{y(age) = \alpha + \beta y^{S}(age)}
where \deqn{y(age) = \frac{1}{2} ln[\frac{q(age)}{1-q(age)}]}
and subscript S defines that function is for standard population. To get mx from qx usual formula is used:
\deqn{m(age) = \frac{q(age)}{n-q(age)(n-a(age))}} where n is the size of age interval and a(x) is a parameter from life table.
}
}
\examples{
age <- seq(40, 80, 10)
mx <- c(0.003, 0.005, 0.009, 0.018, 0.036)
standard.mx <- c(0.0025, 0.004, 0.007, 0.014, 0.03)
mort.approx(mx = mx, age = age, model = "Gompertz")
mort.approx(mx = mx, age = age, model = "Brass", standard.mx = standard.mx)
}
\references{
Preston, S. H., Heuveline, P., & Guillot, M. (2001). Demography: Measuring and Modeling Population Processes. Blackwell Publishers.
}
